local GlobalAddonName, LTS = ...

local ADDON_VERSION = "1.5 (24.04.2016)"
--	Curr Legion Build 21531

--[[
TODO

1.5
Major Database review:
- Fixed sources for a lot items
- More removed quests checks
- Added zones info for world drop items
- New source type: "Created" for 'created items from items' and 'items from containers'
Added tab with class sets
BOE scaning now works for guildbank
Bugfixes

1.4
Minor UI changes
Fixed TAB feature for visuals that can't be obtained on current char
Added option for scan boe items ("/wardrobe boe")
Added "Favorite" text on item tooltips for favorites visuals
Added collection state to tooltips in profession window
Added zones names for vendors (as result - vendors now available for "current zone" filter)
Added missed alliance visuals

1.3
More pre-cata quest checks (this time for items that can be obtained post-wotlk)
Opening window with items & sources on pressing TAB button while you hover item with visual
Option for fading out items that already collected in auction window [Must be enabled in options before using]. Note: Do not forget to check all sources (press TAB buton) before spending gold 
Fixed sources for some items (375 items from BC raids, ICC, Cata T11 were incorectly shown as vendor items)

1.2
DB updated to last Legion Alpha build
Added pre-cata quests to DB
Fixed models preview with different UI scale
Added new filter "Current zone": shows all items from your current instance, raid or quest zone
Added names of chars that collected visual to items tooltips
Added scaling big model by pressing keys "-" and "+"
Added import/export buttons in "about" window
Added "Force update" button in "about" window

1.1
Added favorites
Opening wardrobe before visiting bank or VS isn't requied
Removed legion items
Names of chars that collected visual on model tooltip
List of chars on "about" page
Info on item tooltips about collected state (collected, not collected). Can be disabled in options
Added collected and not collected filter
Hide window on escape press
Fixed WOD crafted items
Added zones names for quests info
]]

local tonumber, GetItemCount, strtrim, bit_band, type = tonumber, GetItemCount, strtrim, bit.band, type

local CategoriesSlots = LTS.CategoriesSlots
local CategoriesSlotsByClass = LTS.CategoriesSlotsByClass
local CamerasRace = LTS.CamerasRace 
local SlotIcons = LTS.SlotIcons
local CameraData = LTS.CameraData
local CategoryBigModelItem = LTS.CategoryBigModelItem
local ItemsCheckBonuses = LTS.ItemsCheckBonuses

local AllData
local AllDataAll
local CamerasData
local Visuals
local ItemSource
local VendorNames
local InstanceList
local BossList
local DiffList
local ItemCost
local QuestToZone
local ZonesNames
local ZoneToDataID
local DiffToDataID
local QuestZoneToDataID
local CategoriesSlotsByClassAll
local NPCToZone
local NPCReact
local SetsAllData
local Sets

local VLTW

local _,playerClass = UnitClass'player'
local _,playerRace = UnitRace'player'
local playerSex = UnitSex'player'
local playerFaction = UnitFactionGroup'player'

CategoriesSlotsByClassAll = CategoriesSlotsByClass.ALL
CategoriesSlotsByClass = CategoriesSlotsByClass[playerClass]

local realmKey = GetRealmName() or ""
local charName = UnitName'player' or ""
realmKey = realmKey:gsub(" ","")
local charKey = charName .. "-" .. realmKey
local charKeyFind = charKey:gsub("%-","%%%-")

local IsDebugVersion = false

local SourceTypeNames = {
	[1] = "Boss Drop",
	[2] = "Quest",
	[3] = "Vendor",
	[4] = "World Drop",
	[5] = "Legacy",
	[6] = "Created",
	[7] = "Achievement",
	[8] = "Profession",
}
local SourceTypeBitFilters = {
	[0] = 0,
	[1] = 1,
	[2] = 2,
	[3] = 4,
	[4] = 8,
	[5] = 16,
	[6] = 32,
	[7] = 64,
	[8] = 128,
}

local majorBonuses = {
	[451] = true,
	[450] = true,
	[566] = true,
	[567] = true,
	[524] = true,
	[525] = true,
	[526] = true,
	[527] = true,
	[593] = true,
	[617] = true,
	[618] = true,
	[558] = true,
	[559] = true,
	[594] = true,
	[619] = true,
	[620] = true,
}

local CameraDataAnimation = {
	--[8] = 15,
	--[14] = 15,
	[1] = 8,
	[11] = 8,
	[12] = 8,
}

if playerRace == "Gnome" and playerSex == 2 then	--Gnome Male Fix
	CameraDataAnimation[10] = 8
	CameraDataAnimation[13] = 8
	CameraDataAnimation[14] = 8
	CameraDataAnimation[15] = 8
	CameraDataAnimation[16] = 8
elseif playerRace == "Pandaren" and playerSex == 2 then	--Pandaren Male Fix
	CameraDataAnimation[15] = 8
end

local CameraDataOffset = 0
local currentPage, currentMaxPages, currentCategory = 1, 1
local CharQuestsData
local IsItemCollected, IsUsedZeroCam, IsFilterEnabled, IsWindowLoaded, IsFavoritesOnly, IsTooltipModDisabled, IsTooltipTabHelpDisabled
local UpdatePage, UpdateCategory, UpdateCamera0, UpdateAlerts, UpdateCollectCategory, UpdateCollectStateBOE, UpdateModel
local eventsFrame, eventsFrame_OnKeyDown
local FilterBits, FilterBitsMax, FilterDebug = 2^#SourceTypeNames-1, 2^#SourceTypeNames-1, 0
local FilterInstance, FilterInstanceDifficulty
local IsCollectedState = 3
local Mod_Tooltips, Mod_Auction, TooltipLastVisual
local FindBonuses
local LocMapNames

local CachedItemToVisual = {}

if not GetBuildInfo():find("^6%.") then
	eventsFrame = CreateFrame("Frame")
	eventsFrame:RegisterEvent("ADDON_LOADED")
	eventsFrame:SetScript("OnEvent",function(self,event,arg1)
		if event == "ADDON_LOADED" then
			if arg1 == GlobalAddonName then
				if type(_G.VLTW)~='table' then
					_G.VLTW = {}
				end
				if _G.VLTW._LegionInfo then
					eventsFrame:UnregisterEvent("ADDON_LOADED")
				end
			elseif arg1 == "Blizzard_Collections" then
				local thxFrame = CreateFrame("Frame",nil,CollectionsJournal)
				thxFrame:SetPoint("TOPLEFT",CollectionsJournal,"BOTTOMLEFT",0,-40)
				thxFrame:SetPoint("TOPRIGHT",CollectionsJournal,"BOTTOMRIGHT",0,-40)
				thxFrame:SetHeight(60)
				
				
				thxFrame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",tile = true,tileSize = 1,edgeSize = 8,insets = {left = 2,right = 2,top = 2,bottom = 2}})
				thxFrame:SetBackdropColor(0, 0, 0, .8) 
				thxFrame:SetBackdropBorderColor(0.6, 0.6, 0.6, 1)
				
				local closeButton = CreateFrame("Button",nil,thxFrame,"UIPanelCloseButton")
				closeButton:SetPoint("TOPRIGHT",4,5)
				
				local thxText = thxFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
				thxText:SetPoint("TOPLEFT",10,-5)
				thxText:SetPoint("BOTTOMRIGHT",-10,5)
				thxText:SetJustifyH("LEFT")
				thxText:SetJustifyV("MIDDLE")
				thxText:SetText("Thanks for using <Legion Wardrobe> addon. Legion invasion already happened and wardrobe+updated transmog system were added to the game. GL & HF in Azeroth.")
				
				_G.VLTW._LegionInfo = true
			end
		end
	end)
	return
end

local function Model_ApplyUICamera(self, uiCameraID, disableOffset, disableAnimation)
	uiCameraID = uiCameraID or 1
	local currCameraData = CamerasData[uiCameraID + (CameraDataOffset * (disableOffset and 0 or 1))] or CamerasData[uiCameraID] or CamerasData[1]
	local posX, posY, posZ, facing = unpack(currCameraData)
	self:MakeCurrentCameraCustom()
	self:SetPosition(posX, posY, posZ)
	self:SetCameraPosition(1, 0, 0)
	self:SetCameraTarget(0, 0, 0)
	self:SetFacing(facing)
	if not disableAnimation then
		self:SetAnimation(CameraDataAnimation[uiCameraID % 20] or 15)
	end
	
	local scale = self:GetEffectiveScale()
	if scale > 0.72 then
		scale = scale/0.7111
		self:SetCameraDistance(scale)
	else
		self:SetCameraDistance(1)
	end
	
	
	self.cameraID = uiCameraID
end

local function GetCursorPos(obj)
	local x_f,y_f = GetCursorPosition()
	local s = obj:GetEffectiveScale()
	x_f, y_f = x_f/s, y_f/s
	local x,y = obj:GetLeft(),obj:GetTop()
	x = x_f-x
	y = (y_f-y)*(-1)
	return x,y
end

local mainFrame = CreateFrame("Frame",nil,UIParent,"InsetFrameTemplate")
mainFrame:SetPoint("CENTER")
mainFrame:SetSize(700,600)
mainFrame:EnableMouse(true)
mainFrame:SetMovable(true)
mainFrame:SetClampedToScreen(true)
mainFrame:RegisterForDrag("LeftButton")
mainFrame:SetScript("OnDragStart", function(self) 
	if self:IsMovable() then 
		self:StartMoving() 
	end 
end)
mainFrame:SetScript("OnDragStop", function(self)
	self:StopMovingOrSizing()
end)
mainFrame:SetScript("OnDragStop", function(self)
	self:StopMovingOrSizing()
end)
mainFrame:SetFrameStrata("HIGH")
mainFrame:Hide()

mainFrame.titleText = mainFrame:CreateFontString(nil, "OVERLAY","GameFontWhite", 2)
mainFrame.titleText:SetFont(mainFrame.titleText:GetFont(),12)
mainFrame.titleText:SetPoint("TOP",0,-4)
mainFrame.titleText:SetJustifyH("LEFT")
mainFrame.titleText:SetJustifyV("TOP")
mainFrame.titleText:SetText("Wardrobe")
mainFrame.titleText:SetTextColor(1,.6,0,1)

local function _CreateShadowBack(mainFrame,disableLine)
	local OverlayShadowTopLeft = mainFrame:CreateTexture(nil,"OVERLAY")
	OverlayShadowTopLeft:SetPoint("TOPLEFT",4,-4)
	OverlayShadowTopLeft:SetAtlas("collections-background-shadow-small",true)

	local OverlayShadowTop = mainFrame:CreateTexture(nil,"OVERLAY")
	OverlayShadowTop:SetPoint("TOPLEFT",17,-4)
	OverlayShadowTop:SetPoint("TOPRIGHT",-17,-4)
	OverlayShadowTop:SetAtlas("collections-background-shadow-small",true)
	OverlayShadowTop:SetTexCoord(0.9999,1,0,1)
	
	local OverlayShadowTopRight = mainFrame:CreateTexture(nil,"OVERLAY")
	OverlayShadowTopRight:SetPoint("TOPRIGHT",-4,-4)
	OverlayShadowTopRight:SetAtlas("collections-background-shadow-small",true)
	OverlayShadowTopRight:SetTexCoord(1,0,0,1)

	local OverlayShadowLeft = mainFrame:CreateTexture(nil,"OVERLAY")
	OverlayShadowLeft:SetPoint("TOPLEFT",4,-17)
	OverlayShadowLeft:SetPoint("BOTTOMLEFT",4,17)
	OverlayShadowLeft:SetAtlas("collections-background-shadow-small",true)
	OverlayShadowLeft:SetTexCoord(0,1,0.9999,1)
	
	local OverlayShadowBottomLeft = mainFrame:CreateTexture(nil,"OVERLAY")
	OverlayShadowBottomLeft:SetPoint("BOTTOMLEFT",4,4)
	OverlayShadowBottomLeft:SetAtlas("collections-background-shadow-small",true)
	OverlayShadowBottomLeft:SetTexCoord(0,1,1,0)
	
	local OverlayShadowRight = mainFrame:CreateTexture(nil,"OVERLAY")
	OverlayShadowRight:SetPoint("TOPRIGHT",-4,-17)
	OverlayShadowRight:SetPoint("BOTTOMRIGHT",-4,17)
	OverlayShadowRight:SetAtlas("collections-background-shadow-small",true)
	OverlayShadowRight:SetTexCoord(1,0,0.9999,1)
	
	local OverlayShadowBottomRight = mainFrame:CreateTexture(nil,"OVERLAY")
	OverlayShadowBottomRight:SetPoint("BOTTOMRIGHT",-4,4)
	OverlayShadowBottomRight:SetAtlas("collections-background-shadow-small",true)
	OverlayShadowBottomRight:SetTexCoord(1,0,1,0)
	
	local OverlayShadowBottom = mainFrame:CreateTexture(nil,"OVERLAY")
	OverlayShadowBottom:SetPoint("BOTTOMLEFT",17,4)
	OverlayShadowBottom:SetPoint("BOTTOMRIGHT",-17,4)
	OverlayShadowBottom:SetAtlas("collections-background-shadow-small",true)
	OverlayShadowBottom:SetTexCoord(0.9999,1,1,0)

	if not disableLine then
		local ShadowLineTop = mainFrame:CreateTexture(nil,"BORDER",nil,1)
		ShadowLineTop:SetPoint("TOPLEFT",4,-13)
		ShadowLineTop:SetPoint("BOTTOMRIGHT",mainFrame,"TOPRIGHT",-4,-17)
		ShadowLineTop:SetAtlas("_collections-background-line",true)
		ShadowLineTop:SetHorizTile(true)
	
		local ShadowLineBottom = mainFrame:CreateTexture(nil,"BORDER",nil,1)
		ShadowLineBottom:SetPoint("TOPLEFT",mainFrame,"BOTTOMLEFT",4,17)
		ShadowLineBottom:SetPoint("BOTTOMRIGHT",-4,13)
		ShadowLineBottom:SetAtlas("_collections-background-line",true)
		ShadowLineBottom:SetHorizTile(true)
	end
	
	local ShadowCornerTopLeft = mainFrame:CreateTexture(nil,"BORDER",nil,2)
	ShadowCornerTopLeft:SetPoint("TOPLEFT",4,-4)
	ShadowCornerTopLeft:SetAtlas("collections-background-shadow-large",true)
	
	local ShadowCornerTopRight = mainFrame:CreateTexture(nil,"BORDER",nil,2)
	ShadowCornerTopRight:SetPoint("TOPRIGHT",-4,-4)
	ShadowCornerTopRight:SetAtlas("collections-background-shadow-large",true)
	ShadowCornerTopRight:SetTexCoord(1,0,0,1)
	
	local ShadowCornerBottomLeft = mainFrame:CreateTexture(nil,"BORDER",nil,2)
	ShadowCornerBottomLeft:SetPoint("BOTTOMLEFT",4,4)
	ShadowCornerBottomLeft:SetAtlas("collections-background-shadow-large",true)
	ShadowCornerBottomLeft:SetTexCoord(0,1,1,0)
	
	local ShadowCornerBottomRight = mainFrame:CreateTexture(nil,"BORDER",nil,2)
	ShadowCornerBottomRight:SetPoint("BOTTOMRIGHT",-4,4)
	ShadowCornerBottomRight:SetAtlas("collections-background-shadow-large",true)
	ShadowCornerBottomRight:SetTexCoord(1,0,1,0)

	local ShadowCornerTop = mainFrame:CreateTexture(nil,"BORDER",nil,2)
	ShadowCornerTop:SetPoint("TOPLEFT",149,-4)
	ShadowCornerTop:SetPoint("TOPRIGHT",-149,-4)
	ShadowCornerTop:SetAtlas("collections-background-shadow-large",true)
	ShadowCornerTop:SetTexCoord(0.9999,1,0,1)	
	
	local ShadowCornerLeft = mainFrame:CreateTexture(nil,"BORDER",nil,2)
	ShadowCornerLeft:SetPoint("TOPLEFT",4,-151)
	ShadowCornerLeft:SetPoint("BOTTOMLEFT",4,151)
	ShadowCornerLeft:SetAtlas("collections-background-shadow-large",true)
	ShadowCornerLeft:SetTexCoord(0,1,0.9999,1)

	local ShadowCornerRight = mainFrame:CreateTexture(nil,"BORDER",nil,2)
	ShadowCornerRight:SetPoint("TOPRIGHT",-4,-151)
	ShadowCornerRight:SetPoint("BOTTOMRIGHT",-4,151)
	ShadowCornerRight:SetAtlas("collections-background-shadow-large",true)
	ShadowCornerRight:SetTexCoord(1,0,0.9999,1)

	local ShadowCornerBottom = mainFrame:CreateTexture(nil,"BORDER",nil,2)
	ShadowCornerBottom:SetPoint("BOTTOMLEFT",149,4)
	ShadowCornerBottom:SetPoint("BOTTOMRIGHT",-149,4)
	ShadowCornerBottom:SetAtlas("collections-background-shadow-large",true)
	ShadowCornerBottom:SetTexCoord(0.9999,1,1,0)
end
do
	_CreateShadowBack(mainFrame)

	local BackgroundTile = mainFrame:CreateTexture(nil,"BACKGROUND")
	BackgroundTile:SetPoint("TOPLEFT",4,-4)
	BackgroundTile:SetPoint("BOTTOMRIGHT",-4,4)
	BackgroundTile:SetAtlas("collections-background-tile",true)
	BackgroundTile:SetVertTile(true)
	BackgroundTile:SetHorizTile(true)
end

local SlotsFrame = CreateFrame("Frame",nil,mainFrame)
SlotsFrame:SetSize(10,10)
SlotsFrame:SetPoint("TOPLEFT",18,-25)

do
	local lastButton = nil
	local buttonsList = {}
	local function ButtonOnClick(self)
		for i=1,#buttonsList do
			buttonsList[i].selected:Hide()
		end
		self.selected:Show()
		UpdateCategory(self._id)
	end
	local function ButtonOnEnter(self)
		if not self.tooltip then return end
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:AddLine(self.tooltip)
		GameTooltip:Show()
	end
	local function ButtonOnLeave(self)
		GameTooltip_Hide()
	end
	for i=1,#CategoriesSlotsByClass do
		local CategoryID = CategoriesSlotsByClass[i]
		local CategoryName = CategoriesSlots[CategoryID]
	
		local button = CreateFrame("Button",nil,SlotsFrame)
		if lastButton then
			if i == 21 then
				button:SetPoint("TOP", lastButton, "BOTTOM", 0, -5)
			elseif i > 21 then
				button:SetPoint("RIGHT", lastButton, "LEFT", -2, 0)
			else
				button:SetPoint("LEFT", lastButton, "RIGHT", 2, 0)
			end
		else
			button:SetPoint("TOPLEFT")
		end
		button:SetSize(31,31)
		button:SetScript("OnClick",ButtonOnClick)
		button:SetScript("OnEnter",ButtonOnEnter)
		button:SetScript("OnLeave",ButtonOnLeave)
		button._id = CategoryID
		button.tooltip = CategoryName
		lastButton = button
		buttonsList[#buttonsList + 1] = button
		
		local normalTexture = button:CreateTexture(nil,"BACKGROUND")
		normalTexture:SetAllPoints()
		normalTexture:SetTexture([[Interface\AddOns\LegionWardrobe\Transmogrify]])
		local iconTCoord = SlotIcons[ CategoryID ]
		normalTexture:SetTexCoord(iconTCoord[1]/512,iconTCoord[2]/512,iconTCoord[3]/512,iconTCoord[4]/512)
		
		local highlightTexture = button:CreateTexture(nil,"BACKGROUND")
		highlightTexture:SetSize(31,31)
		highlightTexture:SetAtlas("bags-roundhighlight")
		highlightTexture:SetPoint("CENTER",0,1)
		
		local selectedTexture = button:CreateTexture(nil,"OVERLAY")
		selectedTexture:SetSize(45,45)
		selectedTexture:SetTexture([[Interface\AddOns\LegionWardrobe\Transmogrify]])
		selectedTexture:SetPoint("CENTER")
		local iconTCoord = SlotIcons[ -1 ]
		selectedTexture:SetTexCoord(iconTCoord[1]/512,iconTCoord[2]/512,iconTCoord[3]/512,iconTCoord[4]/512)
		selectedTexture:SetShown(i==1)
		button.selected = selectedTexture
		
		button:SetNormalTexture(normalTexture) 
		button:SetHighlightTexture(highlightTexture,"ADD") 
	end
	
	
	local bigModelButton = CreateFrame("Button",nil,SlotsFrame)
	bigModelButton:SetPoint("TOPLEFT", buttonsList[1], "BOTTOMLEFT", 0, -5)
	bigModelButton:SetSize(31,31)
	bigModelButton:SetScript("OnClick",function(self)
		if mainFrame.bigModel:IsShown() then
			mainFrame.bigModel:Hide()
			self.Texture:SetDesaturated(true)
		else
			mainFrame.bigModel:Show()
			self.Texture:SetDesaturated(false)
		end
	end)
	bigModelButton:SetScript("OnEnter",ButtonOnEnter)
	bigModelButton:SetScript("OnLeave",ButtonOnLeave)
	bigModelButton.tooltip = "Open Big Model"
	
	bigModelButton.Texture = bigModelButton:CreateTexture(nil,"BACKGROUND")
	bigModelButton.Texture:SetAllPoints()
	bigModelButton.Texture:SetTexture([[Interface\AddOns\LegionWardrobe\Transmogrify]])
	bigModelButton.Texture:SetTexCoord(450/512,482/512,88/512,120/512)
	bigModelButton.Texture:SetDesaturated(true)
end

local ProgressBar = CreateFrame("StatusBar",nil,mainFrame)
ProgressBar:SetSize(196,13)
ProgressBar:SetPoint("TOP",0,-70)
ProgressBar:SetStatusBarTexture([[Interface\PaperDollInfoFrame\UI-Character-Skills-Bar]]) 
ProgressBar:SetStatusBarColor(0.03125,0.85,0.0)

do
	local borderLeft = ProgressBar:CreateTexture(nil,"OVERLAY")
	borderLeft:SetPoint("LEFT",-3,0)
	borderLeft:SetSize(8,17)
	borderLeft:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-Skills-BarBorder]])
	borderLeft:SetTexCoord(0.007843,0.043137,0.193548,0.774193)

	local borderRight = ProgressBar:CreateTexture(nil,"OVERLAY")
	borderRight:SetPoint("RIGHT",4,0)
	borderRight:SetSize(8,17)
	borderRight:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-Skills-BarBorder]])
	borderRight:SetTexCoord(0.043137,0.007843,0.193548,0.774193)

	local borderMid = ProgressBar:CreateTexture(nil,"OVERLAY")
	borderMid:SetPoint("TOPLEFT",borderLeft,"TOPRIGHT")
	borderMid:SetPoint("BOTTOMRIGHT",borderRight,"BOTTOMLEFT")
	borderMid:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-Skills-BarBorder]])
	borderMid:SetTexCoord(0.113726,0.1490196,0.193548,0.774193)
	
	local Background = ProgressBar:CreateTexture(nil,"BACKGROUND",nil,-1)
	Background:SetAllPoints()
	Background:SetTexture(0,0,0,1)
	
	local text = ProgressBar:CreateFontString(nil,"OVERLAY","TextStatusBarText",1)
	text:SetPoint("CENTER",0,0)
	text:SetSize(0,9)
	text:SetJustifyH("CENTER")
	ProgressBar.text = text
end


local closeButton = CreateFrame("Button",nil,mainFrame,"UIPanelCloseButton")
closeButton:SetPoint("TOPRIGHT",4,5)

do
	local model = CreateFrame("DressUpModel",nil,mainFrame)
	model:SetPoint("TOPRIGHT",mainFrame,"TOPLEFT",-10,0)
	
	mainFrame.bigModel = model
	
	model:SetSize(mainFrame:GetHeight() * 90 / 130,mainFrame:GetHeight())
	model:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8X8",edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",tile = true,tileSize = 1,edgeSize = 8,insets = {left = 2,right = 2,top = 2,bottom = 2}})
	model:SetBackdropColor(0.07, 0.07, 0.07, 1) 
	model:SetBackdropBorderColor(0.4, 0.4, 0.4, 1) 
	
	model:MakeCurrentCameraCustom()
	model.cameraID = 1
	
	model:Hide()
	
	function UpdateCamera0()
		local cx,cy,cz = model:GetPosition()
		local cf = model:GetFacing()
		
		IsUsedZeroCam = true
		
		CamerasData[0] = CamerasData[0] or {}
		CamerasData[0][1] = cx
		CamerasData[0][2] = cy
		CamerasData[0][3] = cz
		CamerasData[0][4] = cf
		
		for i=1,#mainFrame.models do
			Model_ApplyUICamera(mainFrame.models[i], 0, true, true)
		end
		if IsDebugVersion then
			print(cx,cy,cz,cf)
		end
	end
	
	model:EnableMouse(true)
	model:SetScript("OnMouseDown",function (self,button)
		local x,y = GetCursorPos(self)
		self.x = x
		self.y = y
		local cx,cy,cz = self:GetPosition()
		self.cx = cx
		self.cy = cy
		self.cz = cz
		self.cf = self:GetFacing()
		self.mov = true
		self.isRotation = button == "RightButton"
		
		self.TextInfo:Hide()
	end)
	model:SetScript("OnMouseUp",function (self)
		UpdateCamera0()
		self.mov = nil
	end)
	model:SetScript("OnUpdate",function (self)
		if not self.mov then
			return
		end
		local x,y = GetCursorPos(self)
		local diff_x = x - self.x
		local diff_y = y - self.y
		if self.isRotation then
			self:SetFacing(self.cf + diff_x / 65)
		else
			self:SetPosition(self.cx,self.cy + diff_x / 200,self.cz - diff_y / 200)
		end
		UpdateCamera0()
	end)
	model:SetScript("OnMouseWheel",function (self,delta)
		local cx,cy,cz = self:GetPosition()
		self:SetPosition(cx + delta / 20,cy,cz)
		UpdateCamera0()
		
		self.TextInfo:Hide()
	end)
	model:SetScript("OnModelLoaded",function (self)
		Model_ApplyUICamera(self, self.cameraID)
	end)
	
	local function Reload(self)
		self:SetUnit("player")
		self:Undress()
		self:TryOn(self.itemID or "item:94984")
		
		Model_ApplyUICamera(self, self.cameraID)
	end
	model.Reload = Reload
	
	model:SetScript("OnShow",function (self)
		self:Reload()
	end)
	
	model.TextInfo = model:CreateFontString(nil, "OVERLAY","GameFontWhite", 1)
	model.TextInfo:SetFont(model.TextInfo:GetFont(),11)
	model.TextInfo:SetPoint("TOPLEFT",4,-4)
	model.TextInfo:SetJustifyH("LEFT")
	model.TextInfo:SetJustifyV("TOP")
	model.TextInfo:SetText("Left Click+Move - moving\nRight Click+Move - rotation\nMouse Wheel - scaling")
end

do
	local alert = CreateFrame("Frame",nil,mainFrame)
	alert:SetSize(1,1)
	alert:SetPoint("BOTTOMLEFT",20,15)
	alert:Hide()
	
	alert.icon = alert:CreateTexture(nil,"BACKGROUND")
	alert.icon:SetSize(24,24)
	alert.icon:SetPoint("BOTTOMLEFT",0,0)
	alert.icon:SetTexture("Interface\\DialogFrame\\UI-Dialog-Icon-AlertNew")
	
	alert.text = alert:CreateFontString(nil, "ARTWORK","GameFontWhite")
	alert.text:SetPoint("BOTTOMLEFT",26,0)
	alert.text:SetHeight(24)
	alert.text:SetJustifyH("LEFT")
	alert.text:SetJustifyV("MIDDLE")
	alert.text:SetText("")
	
	function UpdateAlerts(showVSalert, showBankAlert)
		if showVSalert and showBankAlert then
			alert:Show()
			alert.text:SetText("Void Storage data isn't collected; Bank data isn't collected (for WoD items)")
		elseif showVSalert then
			alert:Show()
			alert.text:SetText("Void Storage data isn't collected")
		elseif showBankAlert then
			alert:Show()
			alert.text:SetText("Bank data isn't collected (for WoD items)")
		else
			alert:Hide()
		end
	end
end

do
	local aboutFrameBack = CreateFrame("Button",nil,mainFrame)
	aboutFrameBack:Hide()

	aboutFrameBack:SetPoint("TOPLEFT")
	aboutFrameBack:SetPoint("BOTTOMRIGHT")
	
	aboutFrameBack.Texture = aboutFrameBack:CreateTexture(nil,"OVERLAY",nil,3)
	aboutFrameBack.Texture:SetAllPoints()
	aboutFrameBack.Texture:SetTexture(0,0,0,.8)
	
	aboutFrameBack:SetScript("OnClick",function() end)
	
	aboutFrameBack:SetFrameStrata("DIALOG")

	local aboutFrame = CreateFrame("Frame",nil,aboutFrameBack)
	aboutFrame:SetPoint("CENTER")
	local ABOUT_FRAME_HEIGHT = 370
	aboutFrame:SetSize(650,ABOUT_FRAME_HEIGHT)
	
	aboutFrame:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8X8",edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",tile = true,tileSize = 1,edgeSize = 8,insets = {left = 2,right = 2,top = 2,bottom = 2}})
	aboutFrame:SetBackdropColor(0.1, 0.1, 0.1, 1) 
	aboutFrame:SetBackdropBorderColor(0.25, 0.25, 0.25, 1)
	
	local closeButton = CreateFrame("Button",nil,aboutFrame,"UIPanelCloseButton")
	closeButton:SetPoint("TOPRIGHT",3,2)
	closeButton:SetScript("OnClick",function()
		aboutFrameBack:Hide()
	end)
	
	local authorText = aboutFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
	authorText:SetPoint("TOPLEFT",10,-10)
	authorText:SetJustifyH("LEFT")
	authorText:SetJustifyV("TOP")
	authorText:SetShadowColor(0,0,0,1)
	authorText:SetShadowOffset(1,-1)
	authorText:SetText("Author: |cffffffffAfiya @ EU-Howling Fjord")
	authorText:SetTextColor(0,.75,1)

	local versionText = aboutFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
	versionText:SetPoint("TOPLEFT",10,-25)
	versionText:SetJustifyH("LEFT")
	versionText:SetJustifyV("TOP")
	versionText:SetShadowColor(0,0,0,1)
	versionText:SetShadowOffset(1,-1)
	versionText:SetText("Addon Version: |cffffffff"..ADDON_VERSION)		
	versionText:SetTextColor(0,.75,1)
	
	local ciit = aboutFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
	ciit:SetPoint("TOPLEFT",10,-55)
	ciit:SetJustifyH("LEFT")
	ciit:SetJustifyV("TOP")
	ciit:SetShadowColor(0,0,0,1)
	ciit:SetShadowOffset(1,-1)
	ciit:SetText("Collected icons info:")
	ciit:SetTextColor(0,.75,1)
	
	local gie = aboutFrame:CreateTexture(nil,"ARTWORK")
	gie:SetPoint("TOPLEFT",ciit,"BOTTOMLEFT",5,-5)
	gie:SetSize(18,18)
	gie:SetTexture("Interface\\RaidFrame\\ReadyCheck-Ready")
	--gie:SetTexture([[Interface\Garrison\GarrisonMissionUI2]])
	--gie:SetTexCoord(528/1024,602/1024,920/1024,996/1024)

	local giet = aboutFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
	giet:SetPoint("LEFT",gie,"RIGHT",5,0)
	giet:SetHeight(18)
	giet:SetJustifyH("LEFT")
	giet:SetJustifyV("MIDDLE")
	giet:SetShadowColor(0,0,0,1)
	giet:SetShadowOffset(1,-1)
	giet:SetText("You have collected item on current character")	

	local yie = aboutFrame:CreateTexture(nil,"ARTWORK")
	yie:SetPoint("TOPLEFT",gie,"BOTTOMLEFT",0,-2)
	yie:SetSize(18,18)
	yie:SetTexture([[Interface\AddOns\LegionWardrobe\Transmogrify]])
	yie:SetTexCoord(222/512,250/512,127/512,153/512)

	local yiet = aboutFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
	yiet:SetPoint("LEFT",yie,"RIGHT",5,0)
	yiet:SetHeight(18)
	yiet:SetJustifyH("LEFT")
	yiet:SetJustifyV("MIDDLE")
	yiet:SetShadowColor(0,0,0,1)
	yiet:SetShadowOffset(1,-1)
	yiet:SetText("You have collected item on another character *")	

	local nt = aboutFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
	nt:SetPoint("TOP",yie,"BOTTOM",0,-5)
	nt:SetPoint("LEFT",ciit,0,0)
	nt:SetPoint("RIGHT",aboutFrame,-10,0)
	nt:SetJustifyH("LEFT")
	nt:SetJustifyV("TOP")
	nt:SetShadowColor(0,0,0,1)
	nt:SetShadowOffset(1,-1)
	nt:SetText("* Note: Collecting items via quests will unlock only items that can be equipped by current character (for armors it must be of your primary armor type).")
	nt:SetTextColor(0,.85,.85)
	
	local fii = aboutFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
	fii:SetPoint("TOPLEFT",nt,"BOTTOMLEFT",0,-10)
	fii:SetJustifyH("LEFT")
	fii:SetJustifyV("TOP")
	fii:SetShadowColor(0,0,0,1)
	fii:SetShadowOffset(1,-1)
	fii:SetText("Favorites:")	
	fii:SetTextColor(0,.75,1)
	
	local favi = aboutFrame:CreateTexture(nil,"ARTWORK")
	favi:SetPoint("TOPLEFT",fii,"BOTTOMLEFT",5,-5)
	favi:SetSize(18,18)
	favi:SetTexture([[Interface\COMMON\FavoritesIcon]])
	favi:SetTexCoord(1/32,26/32,1/32,26/32)
	
	local favit = aboutFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
	favit:SetPoint("LEFT",favi,"RIGHT",5,0)
	favit:SetHeight(18)
	favit:SetJustifyH("LEFT")
	favit:SetJustifyV("MIDDLE")
	favit:SetShadowColor(0,0,0,1)
	favit:SetShadowOffset(1,-1)
	favit:SetText("Right click on visual will add it to favorites list*. Use filter to view only favorites visuals.")

	local fnt = aboutFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
	fnt:SetPoint("TOP",favi,"BOTTOM",0,-5)
	fnt:SetPoint("LEFT",ciit,0,0)
	fnt:SetPoint("RIGHT",aboutFrame,-10,0)
	fnt:SetJustifyH("LEFT")
	fnt:SetJustifyV("TOP")
	fnt:SetShadowColor(0,0,0,1)
	fnt:SetShadowOffset(1,-1)
	fnt:SetText("* Note: Favorites list is account wide")
	fnt:SetTextColor(0,.85,.85)
	
	local disableOtherChars = CreateFrame("CheckButton",nil,aboutFrame,"UICheckButtonTemplate")  
	disableOtherChars.text:SetText("|cffffffffDisable tracking items from different characters")
	disableOtherChars:SetPoint("TOP",fnt,"BOTTOM",0,-10)
	disableOtherChars:SetPoint("LEFT",aboutFrame,10,0)
	disableOtherChars:SetScript("OnClick",function (self)
		VLTW._Disable = self:GetChecked()
		UpdateCategory(currentCategory)
	end)
		
	mainFrame.disableOtherChars = disableOtherChars
	
	local disableTooltip = CreateFrame("CheckButton",nil,aboutFrame,"UICheckButtonTemplate")  
	disableTooltip.text:SetText("|cffffffffDisable collection state info on item tooltips")
	disableTooltip:SetPoint("TOPLEFT",disableOtherChars,"BOTTOMLEFT",0,5)
	disableTooltip:SetScript("OnClick",function (self)
		VLTW._DisableTooltip = self:GetChecked()
		IsTooltipModDisabled = self:GetChecked()
		if not IsTooltipModDisabled then
			Mod_Tooltips()
		end
	end)
	
	mainFrame.disableTooltip = disableTooltip	
	
	local enableTabButton = CreateFrame("CheckButton",nil,aboutFrame,"UICheckButtonTemplate")  
	enableTabButton.text:SetText("|cff"..format("%02x%02x%02x",0,255*.75,255).."Enable opening window with items & sources on pressing \"TAB\" button while you hover item with visual")
	enableTabButton:SetPoint("TOPLEFT",disableTooltip,"BOTTOMLEFT",0,5)
	enableTabButton:SetScript("OnClick",function (self)
		VLTW._DisableTab = not self:GetChecked()
	end)
	
	mainFrame.enableTabButton = enableTabButton
	
	
	local enableAuction = CreateFrame("CheckButton",nil,aboutFrame,"UICheckButtonTemplate")  
	enableAuction.text:SetText("|cffffffffEnable fade out items that already collected in auction window")
	enableAuction:SetPoint("TOPLEFT",enableTabButton,"BOTTOMLEFT",0,5)
	enableAuction:SetScript("OnClick",function (self)
		VLTW._EnableAuction = self:GetChecked()
		Mod_Auction(not self:GetChecked())
	end)
	
	mainFrame.enableAuction = enableAuction	
	
		
	local cnlt = aboutFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
	cnlt:SetPoint("TOP",enableAuction,"BOTTOM",0,-5)
	cnlt:SetPoint("LEFT",ciit,0,0)
	cnlt:SetPoint("RIGHT",aboutFrame,-10,0)
	cnlt:SetJustifyH("LEFT")
	cnlt:SetJustifyV("TOP")
	cnlt:SetShadowColor(0,0,0,1)
	cnlt:SetShadowOffset(1,-1)
	cnlt:SetText("Collected chars info: ")
	cnlt:SetTextColor(0,.75,1)
	
	local cnl = aboutFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
	cnl:SetPoint("TOPLEFT",cnlt,"BOTTOMLEFT",0,-2)
	cnl:SetPoint("RIGHT",aboutFrame,-10,0)
	cnl:SetJustifyH("LEFT")
	cnl:SetJustifyV("TOP")
	cnl:SetTextColor(.9,.9,.9,1)
	cnl:SetShadowColor(0,0,0,1)
	cnl:SetShadowOffset(1,-1)
	cnl:SetText("")
	cnl:SetFont(cnl:GetFont(),11)
	cnl:SetTextColor(1,.6,0)
	
	aboutFrame:SetScript("OnShow",function(self)
		self:SetScript("OnShow",nil)
		local CharsList = {}
		for visualID, charsData in pairs(VLTW._Collected) do
			local t = {strsplit(":", charsData)}
			for i=1,#t do
				CharsList[ t[i] ] = true
			end
		end
		local CharsListSorted = {}
		for q,w in pairs(CharsList) do
			CharsListSorted[#CharsListSorted+1] = q		
		end
		sort(CharsListSorted)
		local res = ""
		for i=1,#CharsListSorted do
			local charName = CharsListSorted[i]
			local bank,vs = true,true
			if VLTW[charName] then
				if VLTW[charName].visited_Bank then
					bank = nil
				end
				if VLTW[charName].visited_VS then
					vs = nil
				end
			end
			local a1 = (bank or vs) and " (" or ""
			local a2 = (bank or vs) and ")" or ""
			res = res .. charName:gsub("%-"..realmKey,"") .. a1 .. (bank and "-B" or "") .. (bank and vs and "," or "") .. (vs and "-VS" or "") .. a2..", "
		end
		res = res:gsub(", $","")
		cnl:SetText(res)
		self:SetHeight(cnl:GetStringHeight()+10+ABOUT_FRAME_HEIGHT)
	end)
	
	local forceUpdate = CreateFrame("Button",nil,aboutFrame,"UIPanelButtonTemplate")
	forceUpdate:SetPoint("BOTTOMRIGHT",aboutFrame,"BOTTOM",-125, 8)
	forceUpdate:SetSize(120,22)
	forceUpdate:SetText("Force Update")
	forceUpdate.tooltipText = "Saving all collected visuals happens during first window opening per session.\nUse this button to reupdate all visualls state on current character."
	forceUpdate:SetScript("OnClick",function()
		for i=1,#CategoriesSlotsByClass do
			UpdateCollectCategory(CategoriesSlotsByClass[i])
		end
	end)
	
	local scanBOE = CreateFrame("Button",nil,aboutFrame,"UIPanelButtonTemplate")
	scanBOE:SetPoint("BOTTOMRIGHT",aboutFrame,"BOTTOM",-2.5, 8)
	scanBOE:SetSize(120,22)
	scanBOE:SetText("Scan BOE")
	scanBOE.tooltipText = "Use this button or chat command \"/wardrobe boe\" while bank opened to collect data from BOE items.\nNote: BOE data has only manual update.\nHold SHIFT key to clear boe data for current char.\nAlso you can add items form guild bank. Type \"/wardrobe boe\" while it opened to add all visuals in current gb tab."
	scanBOE:SetScript("OnClick",function()
		UpdateCollectStateBOE()
	end)	

	
	local ImportExportFrame = CreateFrame("Frame",GlobalAddonName.."ImportExportFrame",UIParent,"UIPanelDialogTemplate")
	ImportExportFrame:SetSize(500,500)
	ImportExportFrame:SetPoint("CENTER")
	ImportExportFrame:SetFrameStrata("FULLSCREEN")
	ImportExportFrame:SetClampedToScreen(true)
	ImportExportFrame:EnableMouse(true)
	ImportExportFrame:SetMovable(true)
	ImportExportFrame:RegisterForDrag("LeftButton")
	ImportExportFrame:SetScript("OnDragStart", function(self) 
		self:StartMoving() 
	end)
	ImportExportFrame:SetScript("OnDragStop", function(self) 
		self:StopMovingOrSizing() 
	end)
	ImportExportFrame:SetScript("OnShow",function (self)
		self.editBox.EditBox:HighlightText()
		self.editBox.EditBox:SetFocus()
	end)
	ImportExportFrame:Hide()
	
	ImportExportFrame.editBox = CreateFrame("ScrollFrame", GlobalAddonName.."ImportExportFrameEditBox", ImportExportFrame, "InputScrollFrameTemplate")
	ImportExportFrame.editBox:SetSize(470,430)
	ImportExportFrame.editBox:SetPoint("TOP",0,-30)
	ImportExportFrame.editBox.EditBox:SetFontObject("ChatFontNormal")
	ImportExportFrame.editBox.EditBox:SetMaxLetters(0)
	ImportExportFrame.editBox.CharCount:Hide()
	ImportExportFrame.editBox.EditBox:SetWidth(470 - 30)
	ImportExportFrame.editBox.EditBox:SetFont(ImportExportFrame.editBox.EditBox:GetFont(),11)
	do
		local parentStrata = ImportExportFrame:GetFrameStrata()
		local parentLevel = ImportExportFrame:GetFrameLevel()
		ImportExportFrame:SetFrameStrata(parentStrata)
		--ImportExportFrame.editBox.FocusButton:SetFrameLevel(parentLevel + 101)
		ImportExportFrame.editBox.EditBox:SetFrameLevel(parentLevel + 102)
		ImportExportFrame.editBox.ScrollBar:SetFrameLevel(parentLevel + 103)
	
	end
	ImportExportFrame:SetToplevel(true)
	
	local ImportExportFrameAccept = CreateFrame("Button",nil,ImportExportFrame,"UIPanelButtonTemplate")
	ImportExportFrameAccept:SetPoint("BOTTOM",0,10)
	ImportExportFrameAccept:SetSize(120,22)
	ImportExportFrameAccept:SetText(ACCEPT)
	ImportExportFrameAccept:SetScript("OnClick",function()
		if not ImportExportFrame.isExport then
			local text_data = strtrim(ImportExportFrame.editBox.EditBox:GetText())
			if text_data == "" then
				ImportExportFrame:Hide()
				return
			end
			local data = {strsplit(";", text_data)}
			wipe(VLTW._Collected)
			local cc = 0
			local chars_text = data[#data]
			local chars = {strsplit(",", chars_text)}
			local ckey = {}
			for i=1,#chars do
				local charName,charKey = chars[i]:match("^([^=]+)=([0-9]+)$")
				ckey[charKey] = charName
			end
			for i=1,#data-1 do
				local vID,vD = data[i]:match("^([0-9]+)=([^=]+)$")
				local vC = {strsplit(",", vD)}
				vID = tonumber(vID)
				local str = ""
				for j=1,#vC do
					str = str .. ckey[ vC[j] ] .. ":"
				end
				str = str:gsub(":$","")
				VLTW._Collected[vID] = str
				cc = cc + 1
			end
			print("<Legion Wardrobe>: Imported "..cc.." visuals")
			for i=1,#CategoriesSlotsByClass do
				UpdateCollectCategory(CategoriesSlotsByClass[i])
			end
			UpdateCategory(currentCategory)
		end
		ImportExportFrame.editBox.EditBox:SetText("")
		ImportExportFrame:Hide()
	end)
	
	local exportButton = CreateFrame("Button",nil,aboutFrame,"UIPanelButtonTemplate")
	exportButton:SetPoint("BOTTOMLEFT",aboutFrame,"BOTTOM",2.5,8)
	exportButton:SetSize(120,22)
	exportButton:SetText("Export")
	exportButton.tooltipText = "If you using addon on different account, but same bnet, that you can import/export all data from one acc to another."
	exportButton:SetScript("OnClick",function()
		local str = ""
		local n = 1
		local cd = {}
		for vid,vd in pairs(VLTW._Collected) do
			local c = {strsplit(":", vd)}
			str = str .. vid .. "="
			for i=1,#c do
				local cn = c[i]
				if not cd[ cn ] then
					cd[ cn ] = format("%02d",n)
					n = n + 1
				end
				str = str .. cd[ cn ]
				if i < #c then
					str = str .. ","
				end
			end
			str = str .. ";"
		end
		for c,ck in pairs(cd) do
			str = str .. c .. "=" .. ck .. ","
		end
		str = str:gsub(",$","")
		ImportExportFrame.editBox.EditBox:SetText(str)
		ImportExportFrame.isExport = true
		ImportExportFrame:Show()
	end)
	
	local importButton = CreateFrame("Button",nil,aboutFrame,"UIPanelButtonTemplate")
	importButton:SetPoint("BOTTOMLEFT",aboutFrame,"BOTTOM",125,8)
	importButton:SetSize(120,22)
	importButton:SetText("Import")
	importButton.tooltipText = "If you using addon on different account, but same bnet, that you can import/export all data from one acc to another."
	importButton:SetScript("OnClick",function()
		StaticPopupDialogs["LW_IMPORT"] = {
			text = "All data about collected items will be rewritten. Are you sure?",
			button1 = YES,
			button2 = NO,
			OnAccept = function()
				ImportExportFrame.editBox.EditBox:SetText("")
				ImportExportFrame.isExport = false
				ImportExportFrame:Show()
			end,
			timeout = 0,
			whileDead = true,
			hideOnEscape = true,
			preferredIndex = 3,
		}
		StaticPopup_Show("LW_IMPORT")
	end)

	
	local function ButtonOnEnter(self)
		if not self.tooltip then return end
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:AddLine(self.tooltip)
		GameTooltip:Show()
	end
	local function ButtonOnLeave(self)
		GameTooltip_Hide()
	end
	local function ButtonOnClick(self)
		aboutFrameBack:Show()
	end

	local button = CreateFrame("Button",nil,mainFrame)
	button:SetPoint("BOTTOMRIGHT",-20,13)
	button:SetSize(31,31)
	button:SetScript("OnClick",ButtonOnClick)
	button:SetScript("OnEnter",ButtonOnEnter)
	button:SetScript("OnLeave",ButtonOnLeave)
	button.tooltip = "About & Help & Options"
	
	local normalTexture = button:CreateTexture(nil,"BACKGROUND")
	normalTexture:SetAllPoints()
	normalTexture:SetTexture([[Interface\AddOns\LegionWardrobe\Transmogrify]])
	normalTexture:SetTexCoord(191/512,220/512,127/512,156/512)
	
	local highlightTexture = button:CreateTexture(nil,"BACKGROUND")
	highlightTexture:SetSize(31,31)
	highlightTexture:SetAtlas("bags-roundhighlight")
	highlightTexture:SetPoint("CENTER",0,1)
	
	button:SetNormalTexture(normalTexture) 
	button:SetHighlightTexture(highlightTexture,"ADD") 
end


do

	local FilterButton = CreateFrame("Button",nil,mainFrame,"UIMenuButtonStretchTemplate")
	FilterButton:SetPoint("BOTTOMRIGHT",-85,20)
	FilterButton:SetText(FILTER)
	FilterButton:SetSize(93,22)
	
	local FilterButtonIcon = FilterButton:CreateTexture(nil,"ARTWORK")
	FilterButtonIcon:SetPoint("RIGHT",-5,0)
	FilterButtonIcon:SetSize(10,12)
	FilterButtonIcon:SetTexture([[Interface\ChatFrame\ChatFrameExpandArrow]])

	local FilterDropDown = CreateFrame("Button",GlobalAddonName.."FilterDropDown",mainFrame,"UIDropDownMenuTemplate")
	UIDropDownMenu_Initialize(FilterDropDown, function(self, level)
		local info = UIDropDownMenu_CreateInfo()
		--info.keepShownOnClick = true
		if level == 1 then
			info.hasArrow = false
			info.isNotRadio = true
			info.notCheckable = false
			
			info.text = FAVORITES
			info.func = function(_, _, _, value)
				if value then
					IsFavoritesOnly = false
				else
					IsFavoritesOnly = true
				end
				UpdateCategory(currentCategory)
			end
			info.checked = function() return IsFavoritesOnly end
			UIDropDownMenu_AddButton(info, level)
			
			info.text = COLLECTED
			info.func = function(_, _, _, value)
				if value then
					IsCollectedState = bit.bxor(IsCollectedState,1)
				else
					IsCollectedState = bit.bor(IsCollectedState,1)
				end
				UpdateCategory(currentCategory)
			end
			info.checked = function() return bit.band(IsCollectedState,1) > 0 end
			UIDropDownMenu_AddButton(info, level)

			info.text = NOT_COLLECTED
			info.func = function(_, _, _, value)
				if value then
					IsCollectedState = bit.bxor(IsCollectedState,2)
				else
					IsCollectedState = bit.bor(IsCollectedState,2)
				end
				UpdateCategory(currentCategory)
			end
			info.checked = function() return bit.band(IsCollectedState,2) > 0 end
			UIDropDownMenu_AddButton(info, level)	
			

			info.text = "Current zone"
			info.func = function(_, _, _, value)
				if FilterInstance then
					FilterInstance = nil
				else
					SetMapToCurrentZone()
					local mapID = GetCurrentMapAreaID()
					if ZoneToDataID[mapID] then
						FilterInstance = ZoneToDataID[mapID]
						local _, _, difficulty = GetInstanceInfo()
						if difficulty then
							FilterInstanceDifficulty = DiffToDataID[difficulty]
						else
							FilterInstanceDifficulty = nil
						end
					elseif QuestZoneToDataID[mapID] then
						FilterInstance = QuestZoneToDataID[mapID]
					else
						print("<Legion Wardrobe>: Filter not added: No data for current zone")
						FilterInstance = nil
					end
				end
				UpdateCategory(currentCategory)
			end
			info.checked = function() return FilterInstance end
			UIDropDownMenu_AddButton(info, level)	
			
			if IsDebugVersion then
				info.text = "Zero Size"
				info.func = function(_, _, _, value)
					if value then
						FilterDebug = bit.bxor(FilterDebug,1)
					else
						FilterDebug = bit.bor(FilterDebug,1)
					end
					UpdateCategory(currentCategory)
				end
				info.checked = function() return bit.band(FilterDebug,1) > 0 end
				UIDropDownMenu_AddButton(info, level)	
				
				info.text = "Boss drop no diff"
				info.func = function(_, _, _, value)
					if value then
						FilterDebug = bit.bxor(FilterDebug,2)
					else
						FilterDebug = bit.bor(FilterDebug,2)
					end
					UpdateCategory(currentCategory)
				end
				info.checked = function() return bit.band(FilterDebug,2) > 0 end
				UIDropDownMenu_AddButton(info, level)	
			end		
			
			info.notCheckable = true

			info.text = CHECK_ALL
			info.func = function()
				FilterBits = 0
				IsFilterEnabled = false
				for _,sbit in pairs(SourceTypeBitFilters) do
					FilterBits = bit.bor(FilterBits,sbit)
				end
				UIDropDownMenu_Refresh(FilterDropDown, 1, 2)
				UpdateCategory(currentCategory)
			end
			UIDropDownMenu_AddButton(info, level)
			
			info.text = UNCHECK_ALL
			info.func = function()
				FilterBits = 0
				IsFilterEnabled = true
				UIDropDownMenu_Refresh(FilterDropDown, 1, 2)
				UpdateCategory(currentCategory)
			end
			UIDropDownMenu_AddButton(info, level)
			info.notCheckable = false

			for i = 1, #SourceTypeNames do
				info.text = SourceTypeNames[i]
				info.func = function(_, _, _, value)
					if value then
						FilterBits = bit.bxor(FilterBits,SourceTypeBitFilters[i])
					else
						FilterBits = bit.bor(FilterBits,SourceTypeBitFilters[i])
					end
					IsFilterEnabled = not (FilterBits == FilterBitsMax)
					UpdateCategory(currentCategory)
				end
				info.checked = function() return bit.band(SourceTypeBitFilters[i],FilterBits) > 0 end
				UIDropDownMenu_AddButton(info, level)
			end
		end
	end, "MENU")

	FilterButton:SetScript("OnClick",function (self)
		PlaySound("igMainMenuOptionCheckBoxOn")
		ToggleDropDownMenu(1, nil, FilterDropDown, self, 74, 15)
	end)
	
end

local inspectScantip = CreateFrame("GameTooltip", GlobalAddonName.."ScanningTooltip", nil, "GameTooltipTemplate")
inspectScantip:SetOwner(UIParent, "ANCHOR_NONE")

local sourcesFrame = CreateFrame("Frame",GlobalAddonName.."SourcesFrame",UIParent,"UIPanelDialogTemplate")
sourcesFrame:SetSize(730,300)
sourcesFrame:SetPoint("CENTER")
sourcesFrame:SetFrameStrata("DIALOG")
sourcesFrame:SetClampedToScreen(true)
sourcesFrame:EnableMouse(true)
sourcesFrame:SetMovable(true)
sourcesFrame:RegisterForDrag("LeftButton")
sourcesFrame:SetScript("OnDragStart", function(self) 
	self:StartMoving() 
end)
sourcesFrame:SetScript("OnDragStop", function(self) 
	self:StopMovingOrSizing() 
end)
sourcesFrame.title:SetText("Sources")
sourcesFrame:SetScript("OnHide", function(self) 
	local prevModel = self.modelFrom
	self.modelFrom = nil
	if prevModel then
		prevModel:GetScript("OnLeave")(prevModel)
	end
end)
sourcesFrame:Hide()

mainFrame.sourcesFrame = sourcesFrame

sourcesFrame.TextLine = sourcesFrame:CreateFontString(nil,"ARTWORK","GameFontWhite")
sourcesFrame.TextLine:SetPoint("TOPLEFT",25,-30)
sourcesFrame.TextLine:SetSize(0,20)
sourcesFrame.TextLine:SetJustifyH("LEFT")
sourcesFrame.TextLine:SetJustifyV("MIDDLE")

sourcesFrame.moreInfo = CreateFrame("Frame",nil,sourcesFrame)
sourcesFrame.moreInfo:Hide()
sourcesFrame.moreInfo:SetPoint("TOPLEFT",sourcesFrame,"TOPRIGHT",0,-25)
sourcesFrame.moreInfo:SetSize(220,300)
sourcesFrame.moreInfo:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8X8",edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",tile = true,tileSize = 1,edgeSize = 8,insets = {left = 2,right = 2,top = 2,bottom = 2}})
sourcesFrame.moreInfo:SetBackdropColor(0, 0, 0, 0.95) 
sourcesFrame.moreInfo:SetBackdropBorderColor(0.6, 0.6, 0.6, 1) 

local function SourcesFrame_EditBox_OnEscapePressed(self)
	self:ClearFocus()
end
local function SourcesFrame_HideTooltip(self)
	GameTooltip_Hide()
end
local function SourcesFrame_ItemName_OnEnter(self)
	if not self.link then return end
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetHyperlink(self.link)
	GameTooltip:Show()
end

local function SourcesFrame_ItemName_OnEvent(self)
	if not self:IsShown() or not self.sourceLink then
		return
	end
	local name,link = GetItemInfo(self.sourceLink)
	if not link then
		return
	end
	self.link = link
	self:GetParent().itemName:SetText(link)
end

local function SourcesFrame_SourceData_OnEnter(self)
	if not self.type then return end
	
	local FirstLine = self:GetParent().sourceData:IsTruncated() and self:GetParent().sourceData:GetText()
		
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	if self.type == 2 then
		if FirstLine then
			GameTooltip:AddLine(FirstLine)
		end
		GameTooltip:AddLine(self.sourceType)
		for i=2,#self.quests do
			local questID = self.quests[i]
			local questName
			inspectScantip:SetHyperlink("quest:"..questID)
			if inspectScantip:NumLines() > 0 then
				local tooltipLine = _G[GlobalAddonName.."ScanningTooltipTextLeft1"]
				local text = tooltipLine:GetText()
				if text and text ~= "" then
					questName = text
				end
			end
			inspectScantip:ClearLines()
			local zoneName = QuestToZone[questID] and ZonesNames[ QuestToZone[questID] ] or ""
			zoneName = zoneName == "" and zoneName or (zoneName..": ")
			if questName then
				GameTooltip:AddLine(zoneName..questName)
			else
				GameTooltip:AddLine(zoneName.."Quest: "..questID)
			end
		end
	elseif self.type == 7 or self.type == 8 then
		GameTooltip:SetHyperlink(self.link)
	elseif self.type == 3 then
		if FirstLine then
			GameTooltip:AddLine(FirstLine)
		end
		GameTooltip:AddLine("Cost")
		local data = self.data
		if data[1] == 1 then
			local amount = GetCoinTextureString(data[2])
			GameTooltip:AddLine(amount)
		elseif data[1] == 2 then
			for i=2,#data,2 do
				local name, _, texturePath = GetCurrencyInfo(data[i])
				GameTooltip:AddLine(name..": "..data[i+1])
				GameTooltip:AddTexture(texturePath)
			end
		elseif data[1] == 3 then
			for i=2,#data,2 do
				local _, link, _, _, _, _, _, _, _, texturePath = GetItemInfo(data[i])
				if link then
					GameTooltip:AddLine(link..": "..data[i+1])
					GameTooltip:AddTexture(texturePath)
				end
			end
		end
	elseif self.type == 6 then
		GameTooltip:SetHyperlink(self.link)
		local firstLine = GameTooltipTextLeft1:GetText()
		if firstLine ~= "" then
			GameTooltipTextLeft1:SetText( "|cffffffff"..(self.type2 and "Contained in" or "Created by").."|r|n"..firstLine )
		end
	else
		if FirstLine then
			GameTooltip:AddLine(FirstLine)
		end
		GameTooltip:AddLine(self.sourceType)
		GameTooltip:AddLine(self.instanceType == 2 and "Raid" or "Instance")
		GameTooltip:AddLine("Zone: |cffffffff"..InstanceList[self.instanceID].."|r")
		GameTooltip:AddLine("Boss: |cffffffff"..BossList[self.bossID].."|r")
		if #self.diffList > 1 then
			GameTooltip:AddLine("Difficulties:")
			for i=1,#self.diffList do
				GameTooltip:AddLine("- |cffffffff"..DiffList[tonumber(self.diffList[i])].."|r")
			end
		elseif self.diffList[1] == "" then
			--nothing
		else
			GameTooltip:AddLine("Difficulty: |cffffffff"..DiffList[tonumber(self.diffList[1])].."|r")
		end
	end
	GameTooltip:Show()
end


local InputFrame = nil
local function ShowInputFrame(title,inputText)
	if not InputFrame then
		InputFrame = CreateFrame("Frame",GlobalAddonName.."ShowInputFrame",UIParent,"UIPanelDialogTemplate")
		InputFrame:SetSize(500,100)
		InputFrame:SetPoint("CENTER")
		InputFrame:SetFrameStrata("FULLSCREEN")
		InputFrame:SetClampedToScreen(true)
		InputFrame:EnableMouse(true)
		InputFrame:SetMovable(true)
		InputFrame:RegisterForDrag("LeftButton")
		InputFrame:SetScript("OnDragStart", function(self) 
			self:StartMoving() 
		end)
		InputFrame:SetScript("OnDragStop", function(self) 
			self:StopMovingOrSizing() 
		end)
		InputFrame:SetScript("OnShow",function (self)
			self.EditBox:HighlightText()
			self.EditBox:SetFocus()
		end)
		InputFrame:Hide()
	
		InputFrame.EditBox = CreateFrame("EditBox",GlobalAddonName.."ShowInputFrameEditBox",InputFrame,"InputBoxTemplate")
		InputFrame.EditBox:SetSize(470,20)
		InputFrame.EditBox:SetPoint("TOP",5,-35)
		InputFrame.EditBox:SetCursorPosition(0)
		InputFrame.EditBox:SetAutoFocus( false )
		InputFrame.EditBox:SetScript("OnEscapePressed",function(self) 
			self:ClearFocus() 
			self:GetParent().OK:Click("LeftButton")
		end)
		InputFrame.EditBox:SetScript("OnEnterPressed",function (self)
			self:GetParent().OK:Click("LeftButton")
		end)
		
		InputFrame.OK = CreateFrame("Button",nil,InputFrame,"UIPanelButtonTemplate")
		InputFrame.OK:SetPoint("TOP",0,-60)
		InputFrame.OK:SetSize(130,22)
		InputFrame.OK:SetText(CLOSE)
		InputFrame.OK:SetScript("OnClick",function (self)
			InputFrame:Hide()
			local input = InputFrame.EditBox:GetText()
		end)
	end
	if InputFrame:IsShown() then
		InputFrame:Hide()
	end
	InputFrame.title:SetText(title or "")
	InputFrame.EditBox:SetText(inputText or "")
	InputFrame:Show()
end


local function SourcesFrame_ItemLink_OnClick(self)
	ShowInputFrame("url",self.link)
end
local function SourcesFrame_MoreInfo_Text_OnEnter(self)
	if not self.type then return end
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	if self.type == 2 then
		GameTooltip:SetHyperlink(self.data)
	elseif self.type == 4 then
		local questID = self.data
		GameTooltip:SetHyperlink("quest:"..questID)
		local zoneName = QuestToZone[questID] and ZonesNames[ QuestToZone[questID] ]
		if zoneName then
			GameTooltip:AddLine("Zone: "..zoneName)
		end
	elseif self.type == 3 then
		local data = self.data[1]
		local npcZone = self.data[2]
		if npcZone then
			local color = nil
			if self.data[3] then
				color = self.data[3] == 1 and "|cffd00000" or "|cff0080ff"
			end
			GameTooltip:AddLine("Zone: "..(color or "")..ZonesNames[ npcZone ]..(color and "|r" or ""))
		end
		GameTooltip:AddLine("Cost")
		if data[1] == 1 then
			local amount = GetCoinTextureString(data[2])
			GameTooltip:AddLine(amount)
		elseif data[1] == 2 then
			for i=2,#data,2 do
				local name, _, texturePath = GetCurrencyInfo(data[i])
				GameTooltip:AddLine(name..": "..data[i+1])
				GameTooltip:AddTexture(texturePath)
			end
		elseif data[1] == 3 then
			for i=2,#data,2 do
				local _, link, _, _, _, _, _, _, _, texturePath = GetItemInfo(data[i])
				if link then
					GameTooltip:AddLine(link..": "..data[i+1])
					GameTooltip:AddTexture(texturePath)
				end
			end
		end
	elseif self.type == 5 then
		GameTooltip:AddLine(self.data)
	elseif self.type == 6 then
		GameTooltip:SetHyperlink(self.data[1])
		local firstLine = GameTooltipTextLeft1:GetText()
		if firstLine ~= "" then
			GameTooltipTextLeft1:SetText( "|cffffffff"..(self.data[2] and "Contained in" or "Created by").."|r|n"..firstLine )
		end
	else
		local data = self.data
		GameTooltip:AddLine(data[1] == 2 and "Raid" or "Instance")
		GameTooltip:AddLine("Zone: |cffffffff"..InstanceList[ data[2] ].."|r")
		GameTooltip:AddLine("Boss: |cffffffff"..BossList[ data[3] ].."|r")
		if #data[4] > 1 then
			GameTooltip:AddLine("Difficulties:")
			for i=1,#data[4] do
				GameTooltip:AddLine("- |cffffffff"..DiffList[tonumber(data[4][i])].."|r")
			end
		elseif data[4][1] == "" then
			--nothing
		else
			GameTooltip:AddLine("Difficulty: |cffffffff"..DiffList[tonumber(data[4][1])].."|r")
		end
	end
	GameTooltip:Show()
end

local function SourcesFrame_ItemName_OnClick(self)
	local link = self.link
	if not link then
		return
	elseif IsModifiedClick("CHATLINK") then
		if ChatEdit_GetActiveWindow() or ( BrowseName and BrowseName:IsVisible() ) or ( TradeSkillFrame and TradeSkillFrame.SearchBox:HasFocus() ) then
			ChatEdit_InsertLink(link)
		else
			ChatFrame_OpenChat(link)
		end
	elseif IsModifiedClick("DRESSUP") then
		DressUpItemLink(link)
	end
end

local SourceFrame_MoreInfo_CreateLine

local function ScheduledItemNameUpdate(self)
	local _, link, _, _, _, _, _, _, _, texture = GetItemInfo(self.itemID)
	if link then
		self.frame:SetText("|T"..texture..":20|t "..link)
	end
end

local function SourcesFrame_MoreInfo_OnClick(self)
	if self.type == 2 then
		--quests
		for i=2,#self.data do
			local questID = self.data[i]
			local questName
			inspectScantip:SetHyperlink("quest:"..questID)
			if inspectScantip:NumLines() > 0 then
				local tooltipLine = _G["LegionWardrobeScanningTooltipTextLeft1"]
				local text = tooltipLine:GetText()
				if text and text ~= "" then
					questName = text
				end
			end
			inspectScantip:ClearLines()
			
			local line = SourceFrame_MoreInfo_CreateLine(i-1)
			if questName then
				line.text:SetText((CharQuestsData[questID] and "|cff00ff00" or "") .. questName)
			else
				line.text:SetText((CharQuestsData[questID] and "|cff00ff00" or "") .."Quest: "..questID)
			end
			line.textFrame.type = 4
			line.textFrame.data = questID
			line.link.link = "http://wowhead.com/quest="..questID
			line:Show()
		end
	elseif self.type == 3 then
		-- vendors
		for i=2,#self.data do
			local npcID = self.data[i]
			local line = SourceFrame_MoreInfo_CreateLine(i-1)
			line.text:SetText(VendorNames[npcID])
			line.link.link = "http://wowhead.com/npc="..npcID
			line.textFrame.type = nil
			if self.itemID  then
				local costTable = ItemCost[ self.itemID ]
				if costTable then
					costTable = costTable[npcID] or costTable
					line.textFrame.type = 3
					line.textFrame.data = {costTable,NPCToZone[npcID],NPCReact[npcID]}
					if costTable[1] == 3 then
						for j=2,#costTable,2 do
							GetItemInfo(costTable[j])
						end
					end
				end
			else
				local npcZone = NPCToZone[npcID]
				if npcZone then
					line.textFrame.type = 5
					local color = nil
					if NPCReact[ npcID ] then
						color = NPCReact[ npcID ] == 1 and "|cffd00000" or "|cff0080ff"
					end
					line.textFrame.data = "Zone: "..(color or "")..ZonesNames[ npcZone ]
				end
			end
			line:Show()
		end
	elseif self.type == 1 then
		-- boss drop
		for i=2,#self.data do
			local instanceType,instanceID,bossID,diffList = self.data[i]:match("^(%d+)#(%d+)#(%d+)#(.*)")
			bossID = tonumber(bossID)
			
			local line = SourceFrame_MoreInfo_CreateLine(i-1)
			line.text:SetText(BossList[bossID])
			
			line.textFrame.type = 1
			line.textFrame.data = {
				tonumber(instanceType),
				tonumber(instanceID),
				bossID,
				{strsplit(":", diffList)},
			}
			line.link.link = "http://wowhead.com/search?q="..BossList[bossID]
			if IsDebugVersion then
				local item = self:GetParent().DATA[3]
				local s = type(item)=='string' and '"' or ''
				line.link.link = "["..s..item..s.."] = "..'"'..self.data[i]..'",'
			end
			line:Show()
		end
	elseif self.type == 8 then
		-- profession
		for i=2,#self.data do
			local spellID = self.data[i]
			local line = SourceFrame_MoreInfo_CreateLine(i-1)
			local spellName = GetSpellInfo(spellID)
			line.text:SetText(spellName)
			line.link.link = "http://wowhead.com/spell="..spellID
			line.textFrame.type = 2
			line.textFrame.data = "spell:"..spellID
			line:Show()
		end
	elseif self.type == 7 then
		-- achievement
		for i=2,#self.data do
			local a4ivID = self.data[i]
			local line = SourceFrame_MoreInfo_CreateLine(i-1)
			local _,a4ivName = GetAchievementInfo(a4ivID)
			line.text:SetText(a4ivName)
			line.link.link = "http://wowhead.com/achievement="..a4ivID
			line.textFrame.type = 2
			line.textFrame.data = GetAchievementLink(a4ivID)
			line:Show()
		end
	elseif self.type == 4 then
		-- world drop
		for i=2,#self.data do
			local zoneID = self.data[i]
			local line = SourceFrame_MoreInfo_CreateLine(i-1)
			line.text:SetText(ZonesNames[ zoneID ])
			line.link.link = "http://wowhead.com/zone="..zoneID
			line.textFrame.type = nil
			line:Show()
		end
	elseif self.type == 6 then
		-- created
		for i=2,#self.data do
			local itemID = self.data[i]
			local line = SourceFrame_MoreInfo_CreateLine(i-1)
			local isContain = false
			if itemID < 0 then
				itemID = -itemID
				isContain = true
			end
			GetItemInfo(itemID)
			line.text:SetText("")
			local s = C_Timer.NewTimer(0.2,ScheduledItemNameUpdate)
			s.itemID = itemID
			s.frame = line.text	
			line.link.link = "http://wowhead.com/item="..itemID
			line.textFrame.data = {"item:"..itemID, isContain}
			line.textFrame.type = 6
			line:Show()
		end
	end
	for i=#self.data,#sourcesFrame.moreInfo.lines do
		sourcesFrame.moreInfo.lines[i]:Hide()
	end
	sourcesFrame.moreInfo:SetHeight(20+(#self.data-1)*25)

	sourcesFrame.moreInfo:Show()
end


sourcesFrame.lines = {}
local function SourceFrame_CreateLine(i)
	if sourcesFrame.lines[i] then
		return sourcesFrame.lines[i]
	end
	local line = CreateFrame("Frame",nil,sourcesFrame)
	sourcesFrame.lines[i] = line
	line:SetPoint("TOPLEFT",10,-30-(i-1)*25-25)
	line:SetSize(1,20)
	
	line.itemName = line:CreateFontString(nil,"ARTWORK","GameFontWhite")
	line.itemName:SetPoint("LEFT",0,0)
	line.itemName:SetSize(180,20)
	line.itemName:SetJustifyH("LEFT")
	line.itemName:SetJustifyV("MIDDLE")
	
	line.itemNameFrame = CreateFrame("Button",nil,line)
	line.itemNameFrame:SetAllPoints(line.itemName)
	line.itemNameFrame:SetScript("OnEnter",SourcesFrame_ItemName_OnEnter)
	line.itemNameFrame:SetScript("OnLeave",SourcesFrame_HideTooltip)
	line.itemNameFrame:SetScript("OnClick",SourcesFrame_ItemName_OnClick)
	line.itemNameFrame:SetScript("OnEvent",SourcesFrame_ItemName_OnEvent)
	line.itemNameFrame:RegisterEvent("GET_ITEM_INFO_RECEIVED")	
	
	line.itemLink = CreateFrame("Button",nil,line,"UIPanelButtonTemplate")
	line.itemLink:SetPoint("LEFT",line.itemName,"RIGHT",5,0)
	line.itemLink:SetSize(50,20)
	line.itemLink:SetText("link")
	line.itemLink:SetScript("OnClick",SourcesFrame_ItemLink_OnClick)

	line.sourceName = line:CreateFontString(nil,"ARTWORK","GameFontWhite")
	line.sourceName:SetPoint("LEFT",line.itemLink,"RIGHT",5,0)
	line.sourceName:SetSize(95,20)
	line.sourceName:SetJustifyH("LEFT")
	line.sourceName:SetJustifyV("MIDDLE")
	
	line.sourceDataAndMore = line:CreateFontString(nil,"ARTWORK","GameFontWhite")
	line.sourceDataAndMore:SetPoint("RIGHT",line.sourceName,"RIGHT",285,0)
	line.sourceDataAndMore:SetSize(0,20)
	line.sourceDataAndMore:SetJustifyH("RIGHT")
	line.sourceDataAndMore:SetJustifyV("MIDDLE")
	line.sourceDataAndMore:SetFont(line.sourceDataAndMore:GetFont(),10)
	
	line.sourceData = line:CreateFontString(nil,"ARTWORK","GameFontWhite")
	line.sourceData:SetPoint("LEFT",line.sourceName,"RIGHT",5,0)
	line.sourceData:SetPoint("RIGHT",line.sourceDataAndMore,"LEFT",-3,0)
	line.sourceData:SetSize(0,20)
	line.sourceData:SetJustifyH("LEFT")
	line.sourceData:SetJustifyV("MIDDLE")
	
	line.sourceDataFrame = CreateFrame("Frame",nil,line)
	line.sourceDataFrame:SetAllPoints(line.sourceData)
	line.sourceDataFrame:SetScript("OnEnter",SourcesFrame_SourceData_OnEnter)
	line.sourceDataFrame:SetScript("OnLeave",SourcesFrame_HideTooltip)
	
	line.moreInfo = CreateFrame("Button",nil,line,"UIPanelButtonTemplate")
	line.moreInfo:SetPoint("LEFT",line.sourceDataAndMore,"RIGHT",5,0)
	line.moreInfo:SetSize(80,20)
	line.moreInfo:SetText("more info")
	line.moreInfo:SetScript("OnClick",SourcesFrame_MoreInfo_OnClick)
	
	if IsDebugVersion and false then
		for i=0,#SourceTypeNames do
			local b = CreateFrame("Button",nil,line,"UIPanelButtonTemplate")
			b:SetPoint("TOPLEFT",line.sourceData,5+i*20,0)
			b:SetSize(20,10)
			b:SetText(i)
			b.tooltipText = SourceTypeNames[i] or "remove"
			b:SetScript("OnClick",function(self)
				local item = self:GetParent().DATA[3]
				local s = type(item)=='string' and '"' or ''
				JJBox("["..s..item..s.."] = "..i..",")
			end)
		end
		line.sourceDataFrame:Hide()
	end
	
	return line
end
sourcesFrame.moreInfo.lines = {}
function SourceFrame_MoreInfo_CreateLine(i)
	if sourcesFrame.moreInfo.lines[i] then
		return sourcesFrame.moreInfo.lines[i]
	end
	local line = CreateFrame("Frame",nil,sourcesFrame.moreInfo)
	sourcesFrame.moreInfo.lines[i] = line
	line:SetPoint("TOPLEFT",10,-10-(i-1)*25)
	line:SetSize(1,20)
	
	line.text = line:CreateFontString(nil,"ARTWORK","GameFontWhite")
	line.text:SetPoint("LEFT",0,0)
	line.text:SetSize(150,20)
	line.text:SetJustifyH("LEFT")
	line.text:SetJustifyV("MIDDLE")
	
	line.textFrame = CreateFrame("Frame",nil,line)
	line.textFrame:SetAllPoints(line.text)
	line.textFrame:SetScript("OnEnter",SourcesFrame_MoreInfo_Text_OnEnter)
	line.textFrame:SetScript("OnLeave",SourcesFrame_HideTooltip)
	
	line.link = CreateFrame("Button",nil,line,"UIPanelButtonTemplate")
	line.link:SetPoint("LEFT",line.text,"RIGHT",5,0)
	line.link:SetSize(50,20)
	line.link:SetText("link")
	line.link:SetScript("OnClick",SourcesFrame_ItemLink_OnClick)
	
	return line
end

local function UpdateLocalizatedSubZones()
	if LocMapNames then
		return
	end
	LocMapNames = {}
	local allZones = {}
	GetAreaMaps(allZones)
	for i=1,#allZones do
		local mapName = GetMapNameByID(allZones[i])
		LocMapNames[ allZones[i] ] = mapName
	end
end

local function ScheduledQuestNameUpdate(s,link,frame)
	if s then
		link = s.link
		frame = s.frame
	end
	inspectScantip:SetHyperlink(link)
	if inspectScantip:NumLines() > 0 then
		local tooltipLine = _G["LegionWardrobeScanningTooltipTextLeft1"]
		local text = tooltipLine:GetText()
		if text and text ~= "" then
			frame:SetText(text)
			inspectScantip:ClearLines()
			return true
		end
	end
	inspectScantip:ClearLines()
	return false
end

local function Model_OnClick(self,button)
	if IsModifiedClick("DRESSUP") and self.Equipped then
		DressUpItemLink(self.Equipped)
		return
	elseif IsModifiedClick("CHATLINK") and self.Equipped then
		local _,link = GetItemInfo(self.Equipped)
		if link then
			if ChatEdit_GetActiveWindow() then
				ChatEdit_InsertLink(link)
			else
				ChatFrame_OpenChat(link)
			end
		end
		return
	elseif button == "RightButton" then
		local isFav = not self.fav
		self.fav = isFav
		self.favIcon:SetShown(isFav)
		if isFav then
			VLTW._Fav[self.visualID] = true
		else
			VLTW._Fav[self.visualID] = nil
		end		
		return
	end
	
	--UpdateLocalizatedSubZones()

	local c = #self.data
	sourcesFrame:SetHeight(40+c*25+25)
	sourcesFrame.TextLine:SetText((self.isCollected and "|cff00ff00Collected|r" or "|cffff0000Not collected|r")..(IsDebugVersion and " VisualID: "..self.visualID or ""))
	for i=1,c do
		local line = SourceFrame_CreateLine(i)
		
		local source = self.data[i]
		line.DATA = source
		
		local name,link = GetItemInfo(source[2])
		line.itemName:SetText(link)
		line.itemLink.link = "http://wowhead.com/item="..source[1]..(source[6] and "&bonus="..source[6] or "")
		line.sourceName:SetText(SourceTypeNames[ source[4] ] or "")
		if source[7] then
			line.sourceName:SetTextColor(0,1,0,1)
		else
			line.sourceName:SetTextColor(1,1,1,1)
		end
		line.itemNameFrame.link = link
		line.itemNameFrame.sourceLink = source[2]
		line.moreInfo:Show()
		line.moreInfo.itemID = source[1]
		line.sourceDataAndMore:SetText("")
		
		if source[4] == 3 and #source[5] > 1 then
			line.moreInfo.data = source[5]
			line.moreInfo.itemID = source[1]
			line.moreInfo.type = 3
			
			local npcID = source[5][2]
			local npcZone = nil
			if NPCToZone[npcID] then
				local color = nil
				if NPCReact[ npcID ] then
					color = NPCReact[ npcID ] == 1 and "|cffd00000" or "|cff0080ff"
				end
				npcZone = " [Zone: "..(color or "")..(ZonesNames[ NPCToZone[npcID] ] or "?")..(color and "|r" or "").."]"
			end
			line.sourceData:SetText(VendorNames[npcID]..(npcZone or ""))
			if #source[5] > 2 then
				line.sourceDataAndMore:SetText("and "..(#source[5] - 2).." more")
			end
			line.sourceDataFrame.type = nil
			local costTable = ItemCost[ source[1] ]
			if costTable then
				costTable = costTable[npcID] or costTable
				line.sourceDataFrame.type = 3
				line.sourceDataFrame.data = costTable
				if costTable[1] == 3 then
					for j=2,#costTable,2 do
						GetItemInfo(costTable[j])
					end
				end
			end
		elseif source[4] == 8 and #source[5] > 1 then
			line.moreInfo.data = source[5]
			line.moreInfo.type = 8
			
			local SpellName = GetSpellInfo(source[5][2])
			line.sourceData:SetText(SpellName)
			line.sourceDataFrame.type = 8
			line.sourceDataFrame.link = "spell:"..source[5][2]
			if #source[5] > 2 then
				line.sourceDataAndMore:SetText("and "..(#source[5] - 2).." more")
			end
		elseif source[4] == 7 and #source[5] > 1 then
			line.moreInfo.data = source[5]
			line.moreInfo.type = 7
			
			local _,a4ivName = GetAchievementInfo(source[5][2])
			line.sourceData:SetText(a4ivName or "")
			line.sourceDataFrame.type = 7
			line.sourceDataFrame.link = GetAchievementLink(source[5][2])
		elseif source[4] == 2 and #source[5] > 1 then
			line.sourceDataFrame.quests = source[5]
			local setted = false
			for j=2,#source[5] do
				if not setted then
					setted = ScheduledQuestNameUpdate(nil,"quest:"..source[5][j],line.sourceData)
				else
					break
				end
			end
			if not setted then
				line.sourceData:SetText("")
				if IsDebugVersion then
					line.sourceData:SetText("quest:"..source[5][2])
				end
				local s = C_Timer.NewTimer(0.5,ScheduledQuestNameUpdate)
				s.link = "quest:"..source[5][2]
				s.frame = line.sourceData
			end
			line.sourceDataFrame.sourceType = SourceTypeNames[ source[4] ]
			line.sourceDataFrame.type = 2
			
			line.moreInfo.data = source[5]
			line.moreInfo.type = 2
			if #source[5] > 2 then
				line.sourceDataAndMore:SetText("and "..(#source[5] - 2).." more")
			end
		elseif source[4] == 1 and #source[5] > 1 then
			local instanceType,instanceID,bossID,diffList = source[5][2]:match("^(%d+)#(%d+)#(%d+)#(.*)")
			instanceType = tonumber(instanceType)
			instanceID = tonumber(instanceID)
			bossID = tonumber(bossID)
			
			line.sourceData:SetText(InstanceList[instanceID]..": "..BossList[bossID])
			if #source[5] > 2 then
				line.sourceDataAndMore:SetText("and "..(#source[5] - 2).." more")
			end
			
			line.sourceDataFrame.instanceType = instanceType
			line.sourceDataFrame.instanceID = instanceID
			line.sourceDataFrame.bossID = bossID
			line.sourceDataFrame.sourceType = SourceTypeNames[ source[4] ]
			line.sourceDataFrame.diffList = {strsplit(":", diffList)}
			line.sourceDataFrame.type = 1
			
			line.moreInfo.data = source[5]
			line.moreInfo.type = 1
		elseif source[4] == 4 and #source[5] > 1 then
			line.moreInfo.data = source[5]
			line.moreInfo.type = 4
			
			line.sourceData:SetText("Zone: "..ZonesNames[ source[5][2] ])
			line.sourceDataFrame.type = nil
			if #source[5] > 2 then
				line.sourceDataAndMore:SetText("and "..(#source[5] - 2).." more")
			end
		elseif source[4] == 6 and #source[5] > 1 then
			local itemID = source[5][2]
			if itemID < 0 then 
				itemID = -itemID 
				line.sourceDataFrame.type2 = true
			else
				line.sourceDataFrame.type2 = nil
			end
			GetItemInfo(itemID)
			line.sourceData:SetText("")
			local s = C_Timer.NewTimer(0.2,ScheduledItemNameUpdate)
			s.itemID = itemID
			s.frame = line.sourceData
			
			line.sourceDataFrame.type = 6
			line.sourceDataFrame.link = "item:"..itemID
			
			line.moreInfo.data = source[5]
			line.moreInfo.type = 6
			if #source[5] > 2 then
				line.sourceDataAndMore:SetText("and "..(#source[5] - 2).." more")
			end
		else
			line.sourceData:SetText("")
			line.sourceDataFrame.type = nil
			line.moreInfo:Hide()
		end
		line:Show()
	end
	for i=c+1,#sourcesFrame.lines do
		sourcesFrame.lines[i]:Hide()
	end
	sourcesFrame.moreInfo:Hide()
	sourcesFrame:Show()
	
	self.border:SetBackdropBorderColor(0, 1, 0, 1)
	local prevModel = sourcesFrame.modelFrom
	if prevModel then
		sourcesFrame.modelFrom = nil
		prevModel:GetScript("OnLeave")(prevModel)
	end
	sourcesFrame.modelFrom = self
end

local function Model_OnModelLoaded(self)
	if self.cameraID then
		Model_ApplyUICamera(self, self.cameraID)
	end
end

local function Model_OnShow(self)
	self:SetUnit("player")
	self:Undress()
	if self.Equipped then
		self:TryOn(self.Equipped)
	end
	if self.cameraID then
		Model_ApplyUICamera(self, self.cameraID)
	end
end

local function Model_OnEnter(self)
	if not self.data then return end
	self.border:SetBackdropBorderColor(1, 1, 1, 1)
	if mainFrame.bigModel:IsShown() and self.Equipped then
		mainFrame.bigModel:Undress()
		mainFrame.bigModel:TryOn(self.Equipped)
	end
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:AddLine(self.isCollected and "|cff00ff00Collected|r" or "|cffff0000Not collected|r")
	for i=1,#self.data do
		local data = self.data[i]
		local name,link = GetItemInfo(data[2])
		GameTooltip:AddDoubleLine(link or " ",(data[7] and "|cff00ff00" or "").. (SourceTypeNames[ data[4] ] or "").."|r")
	end
	if self.isCollected then
		local collStr = VLTW._Collected[self.visualID] or VLTW._BOE[self.visualID]
		GameTooltip:AddLine("Collected by |cffffffff"..collStr:gsub(":",", "):gsub("%-"..realmKey,""), nil, nil, nil, true)
	end
	GameTooltip:Show()
end

local function Model_OnLeave(self)
	if sourcesFrame.modelFrom == self then
		self.border:SetBackdropBorderColor(0, 1, 0, 1)
	elseif self.isCollected then
		self.border:SetBackdropBorderColor(1, 1, 0.5, 1)
	else
		self.border:SetBackdropBorderColor(0.6, 0.6, 0.6, .8)
	end
	GameTooltip:Hide()
end

local function Model_Collected_SetView(self,isYellow)
	if isYellow then
		self:SetTexture([[Interface\AddOns\LegionWardrobe\Transmogrify]])
		self:SetTexCoord(222/512,250/512,127/512,153/512)
		self:SetSize(28,28)
		self:SetPoint("CENTER",self:GetParent(),"TOPLEFT",12,-11)
	else
		--self:SetTexture([[Interface\Garrison\GarrisonMissionUI2]])
		--self:SetTexCoord(528/1024,602/1024,920/1024,996/1024)
		self:SetTexture("Interface\\RaidFrame\\ReadyCheck-Ready")
		self:SetTexCoord(0,1,0,1)
		self:SetSize(30,30)
		self:SetPoint("CENTER",self:GetParent(),"TOPLEFT",10,-5)
	end
end

local function Model_OnEvent(self,event)
	if event == "GET_ITEM_INFO_RECEIVED" then
		if not self:IsShown() or not self.Equipped then
			return
		end
		self:TryOn(self.Equipped)	--For items with bonuses
	else
		self:RefreshCamera()
		if ( self.cameraID ) then
			Model_ApplyUICamera(self, self.cameraID)
		end
	end
end

local models = {}
mainFrame.models = models
for i=0,2 do
	for j=0,5 do
		local borderFrame = CreateFrame("Frame",nil,mainFrame)
	
		local model = CreateFrame("DressUpModel",nil,mainFrame)
		models[i*6+j+1] = model
		model:SetPoint("TOPLEFT",50+100*j,-100-140*i)
		model:SetSize(90,130)
		
		model:SetScript("OnModelLoaded",Model_OnModelLoaded)
		model:SetScript("OnShow",Model_OnShow)
		model:SetScript("OnEnter",Model_OnEnter)
		model:SetScript("OnLeave",Model_OnLeave)
		model:SetScript("OnMouseUp",Model_OnClick)

		model:RegisterEvent("UI_SCALE_CHANGED")
		model:RegisterEvent("DISPLAY_SIZE_CHANGED")
		--model:RegisterEvent("GET_ITEM_INFO_RECEIVED")
		model:SetScript("OnEvent",Model_OnEvent)
		
		model.collected = model:CreateTexture(nil,"OVERLAY")
		model.collected:SetPoint("CENTER",model,"TOPLEFT",10,-5)
		model.collected:SetSize(30,30)
		
		model.favIcon = model:CreateTexture(nil,"OVERLAY")
		model.favIcon:SetPoint("CENTER",model,"TOPRIGHT",-5,-3)
		model.favIcon:SetSize(30,30)	
		model.favIcon:SetTexture([[Interface\COMMON\FavoritesIcon]])
		model.favIcon:SetTexCoord(1/32,26/32,1/32,26/32)	
		
		model.collected.SetView = Model_Collected_SetView
		
		borderFrame:SetPoint("TOPLEFT",model,-4,4)
		borderFrame:SetPoint("BOTTOMRIGHT",model,4,-4)
		
		borderFrame:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8X8",edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",tile = true,tileSize = 1,edgeSize = 8,insets = {left = 2,right = 2,top = 2,bottom = 2}})
		borderFrame:SetBackdropColor(0.08, 0.08, 0.08, 1) 
		borderFrame:SetBackdropBorderColor(0.6, 0.6, 0.6, 1)
		
		model.border = borderFrame

	end
end

local PageText = mainFrame:CreateFontString(nil,"ARTWORK","GameFontWhite")
PageText:SetPoint("BOTTOM",-57,53)
PageText:SetJustifyH("RIGHT")

local PrevPageButton = CreateFrame("Button",nil,mainFrame)
PrevPageButton:SetPoint("BOTTOM",20,43)
PrevPageButton:SetSize(32,32)
PrevPageButton:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up") 
PrevPageButton:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Down") 
PrevPageButton:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Disabled") 
PrevPageButton:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight","ADD") 
PrevPageButton:SetScript("OnClick",function ()
	currentPage = max(1,currentPage - 1)
	UpdatePage()
end)

local NextPageButton = CreateFrame("Button",nil,mainFrame)
NextPageButton:SetPoint("BOTTOM",55,43)
NextPageButton:SetSize(32,32)
NextPageButton:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up") 
NextPageButton:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down") 
NextPageButton:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Disabled") 
NextPageButton:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight","ADD")
NextPageButton:SetScript("OnClick",function ()
	currentPage = min(currentMaxPages,currentPage + 1)
	UpdatePage()
end)

mainFrame:SetScript("OnMouseWheel",function (self,delta)
	if delta < 0 and NextPageButton:IsEnabled() then
		NextPageButton:Click()
	elseif delta > 0 and PrevPageButton:IsEnabled() then
		PrevPageButton:Click()
	end
end)

local function FindVisualAndOpen(visualID)
	local canObtained = false
	for cat,categoryData in pairs(AllData) do
		for i=1,#categoryData do
			if categoryData[i] == visualID then
				canObtained = true
				break
			end
		end
		if canObtained then
			GetQuestsCompleted(CharQuestsData)	--Update Quests Data
			break
		end
	end
	local model = models[1]
	UpdateModel(model, visualID, not canObtained)
	Model_OnClick(model,"LeftButton")
end

function UpdateModel(model,visualID,canNotObtained)
	model:Show()
	model.border:Show()
	model:Undress()
	
	model.data = {}
	model.visualID = visualID
	local visualData = Visuals[ visualID ]
	local isCollected = 0
	if visualData then
		for k=1,#visualData do
			local item = visualData[k]
			local itemID = type(item) == 'number' and item or tonumber(item:match("^(%d+)"))
			local itemLink = "item:"..itemID
			
			local sourceData = ItemSource[item]
			local sourceType = sourceData[1]
			
			if type(item) == 'string' then
				local count = 0
				for j=1,#item do
					if item:sub(j,j) == ":" then
						count = count + 1
					end
				end
				itemLink = itemLink .. ":0:0:0:0:0:0:0:"..UnitLevel'player'..":0:0:0:"..count..":"..item:gsub("^%d+:","")
			end
			
			GetItemInfo(itemLink)	--Cache it
			
			local currItemCollected = false
			if not canNotObtained then
				if IsItemCollected(item) then
					isCollected = 1
					currItemCollected = true
				elseif sourceType == 2 then
					for j=2,#sourceData do
						if CharQuestsData[ sourceData[j] ] then
							isCollected = 1
							currItemCollected = true
							break
						end
					end
				end
			end
			
			model.data[k] = {
				itemID,
				itemLink,
				item,
				sourceType,
				sourceData,
				type(item) == 'string' and item:gsub("^%d+:",""),
				currItemCollected,
			}
			
			if k == 1 then
				model:TryOn(itemLink)
				model.Equipped = itemLink
			end
		end
	else
		--print('Visual',visualID,'not found')
	end
	if isCollected == 0 and (VLTW._Collected[visualID] or VLTW._BOE[visualID]) and not VLTW._Disable then
		isCollected = 2
	end
	if isCollected > 0 then
		model.collected:Show()
		model.collected:SetView(isCollected == 2)
		model.border:SetBackdropBorderColor(1, 1, 0.5, 1)
	else
		model.collected:Hide()
		model.border:SetBackdropBorderColor(0.6, 0.6, 0.6, .8)
	end
	model.isCollected = isCollected > 0 and isCollected or nil
	
	local isFav = VLTW._Fav[visualID]
	model.fav = isFav
	model.favIcon:SetShown(isFav)
end

function UpdatePage()
	GameTooltip_Hide()

	PageText:SetFormattedText(COLLECTION_PAGE_NUMBER, currentPage, currentMaxPages)
	if currentPage == 1 then PrevPageButton:Disable() else PrevPageButton:Enable() end
	if currentPage >= currentMaxPages then NextPageButton:Disable() else NextPageButton:Enable() end

	local pos = (currentPage - 1) * #models + 1
	local m_c = 0
	
	local categoryData = AllData[currentCategory]
	if IsFilterEnabled or IsFavoritesOnly or IsCollectedState < 3 or FilterInstance or FilterDebug > 0 then
		categoryData = AllData[-1]
	end
	
	for i=pos,#categoryData do
		m_c = m_c + 1
		local model = models[m_c]
		
		UpdateModel(model, categoryData[i])
		
		if m_c >= #models then
			break
		end
	end
	for i=m_c+1,#models do
		models[i]:Hide()
		models[i].border:Hide()
	end
	if IsUsedZeroCam then
		UpdateCamera0()
	end
end

local isFirstLoad = true

function UpdateCollectCategory(categoryID)
	GetQuestsCompleted(CharQuestsData)	--Update Quests Data
	local collected = 0
	
	local categoryData = AllData[categoryID]
	if IsFilterEnabled or IsFavoritesOnly or IsCollectedState < 3 or FilterInstance or FilterDebug > 0 then
		categoryData = AllData[-1]
	end
	
	for i=1,#categoryData do
		local visualID = categoryData[i]
		local visualData = Visuals[ visualID ]
		local isCollected = 0
		if visualData then
			for k=1,#visualData do
				local item = visualData[k]
				local sourceData = ItemSource[item]
				if IsItemCollected(item) then
					isCollected = 1
				elseif sourceData[1] == 2 then
					for j=2,#sourceData do
						if CharQuestsData[ sourceData[j] ] then
							isCollected = 1
							break
						end
					end
				end
			end
		end
		if not VLTW._Disable then
			local inDBstr = VLTW._Collected[visualID]
			if isCollected == 1 then
				if not inDBstr then
					VLTW._Collected[visualID] = charKey
				elseif not inDBstr:find(charKeyFind) then
					VLTW._Collected[visualID] = inDBstr..":"..charKey
				end
			elseif inDBstr then
				isCollected = 1
				if inDBstr:find(charKeyFind) then
					inDBstr = strtrim(inDBstr:gsub(":*"..charKeyFind..":*",":"),":")
					if inDBstr == "" then
						VLTW._Collected[visualID] = nil
						isCollected = 0
					else
						VLTW._Collected[visualID] = inDBstr
					end
				end
			end
			
			if isCollected == 0 then
				isCollected = VLTW._BOE[visualID] and 1 or 0
			end
		end
		collected = collected + isCollected
	end
	return collected
end

function UpdateCollectStateBOE()
	if GuildBankFrame and GuildBankFrame:IsShown() and not IsShiftKeyDown() then
		local currentTab = GetCurrentGuildBankTab()
		for i=1,98 do
			local link = GetGuildBankItemLink(currentTab,i)
			if link then
				local itemID = tonumber(link:match("item:(%d+)"))
				if ItemsCheckBonuses[itemID] then
					local bonus = FindBonuses(link)
					if bonus ~= 0 then
						itemID = itemID ..":"..bonus
					end
				end
				
				local visualID = nil
				for currVisualID,visualData in pairs(Visuals) do
					for i=1,#visualData do
						if visualData[i] == itemID then
							visualID = currVisualID
							break
						end
					end
					if visualID then
						break
					end
				end
				
				if visualID and not VLTW._Collected[visualID] then
					local inDBstr = VLTW._BOE[visualID]
					if not inDBstr then
						VLTW._BOE[visualID] = charKey
					elseif not inDBstr:find(charKeyFind) then
						VLTW._BOE[visualID] = inDBstr..":"..charKey
					end
				end
			end
		end
		print("<Legion Wardrobe>: BOE data from guildbank (tab "..currentTab..") added")
		return
	end

	local toRemove = {}
	for visualID,collectedStr in pairs(VLTW._BOE) do
		if collectedStr:find(charKeyFind) then
			collectedStr = strtrim(collectedStr:gsub(":*"..charKeyFind..":*",":"),":")
			if collectedStr == "" then
				toRemove[#toRemove + 1] = visualID
			else
				VLTW._BOE[visualID] = collectedStr
			end
		end
	end
	for i=1,#toRemove do
		VLTW._BOE[ toRemove[i] ] = nil
	end
	if IsShiftKeyDown() then
		print("<Legion Wardrobe>: BOE data for "..charKey.." cleared")
		return
	end
	if GetContainerNumSlots(5) == 0 and GetNumBankSlots() > 0 then
		print("<Legion Wardrobe>: Bank isn't opened")
	end
	for i=-1,11 do
		for j=1,GetContainerNumSlots(i) do
			local link = GetContainerItemLink(i, j)
			if link then
				local isBOE = false
				inspectScantip:SetBagItem(i, j)
				if inspectScantip:NumLines() > 0 then
					local numLines = inspectScantip:NumLines()
					if numLines > 5 then
						numLines = 5
					end
					for k=1,numLines do
						if _G[GlobalAddonName.."ScanningTooltipTextLeft"..k]:GetText() == ITEM_BIND_ON_EQUIP then
							isBOE = true
							break
						end
					end
				end
				inspectScantip:ClearLines()
				if isBOE then
					local itemID = tonumber(link:match("item:(%d+)"))
					if ItemsCheckBonuses[itemID] then
						local bonus = FindBonuses(link)
						if bonus ~= 0 then
							itemID = itemID ..":"..bonus
						end
					end
					
					local visualID = nil
					for currVisualID,visualData in pairs(Visuals) do
						for i=1,#visualData do
							if visualData[i] == itemID then
								visualID = currVisualID
								break
							end
						end
						if visualID then
							break
						end
					end
					
					if visualID and not VLTW._Collected[visualID] then
						local inDBstr = VLTW._BOE[visualID]
						if not inDBstr then
							VLTW._BOE[visualID] = charKey
						elseif not inDBstr:find(charKeyFind) then
							VLTW._BOE[visualID] = inDBstr..":"..charKey
						end
					end
				end
			end
		end
	end
	print("<Legion Wardrobe>: BOE collected")
end

local function CheckFilterInstance(sourceData,type)
	if type == 1 then
		for i=2,#sourceData do
			--2#17#192#3
			local instanceType,instanceDataID,bossDataID,diffDataIDs = sourceData[i]:match("^(%d+)#(%d+)#(%d+)#(.*)")
			local diffFilter = false
			if FilterInstanceDifficulty and diffDataIDs and diffDataIDs ~= "" then
				diffDataIDs = {strsplit(":", diffDataIDs)}
				for j=1,#diffDataIDs do
					if FilterInstanceDifficulty == diffDataIDs[j] then
						diffFilter = true
						break
					end
				end
			else
				diffFilter = true
			end
			if instanceDataID == FilterInstance and diffFilter then
				return true
			end
		end
	elseif type == 2 or type == 3 then
		local zonesTable = (type == 2 and QuestToZone) or (type == 3 and NPCToZone)
		for i=2,#sourceData do
			local zoneID = zonesTable[ sourceData[i] ]
			if zoneID then
				for j=1,#FilterInstance do
					if FilterInstance[j] == zoneID then
						return true
					end
				end
			end
		end
	elseif type == 4 then
		for i=2,#sourceData do
			for j=1,#FilterInstance do
				if FilterInstance[j] == sourceData[i] then
					return true
				end
			end
		end
	end
	return false
end

local function CheckFilterDebugNoDiff(sourceData)
	for i=2,#sourceData do
		if sourceData[i]:match("#$") then
			return true
		end
	end
end

local function CreateFilteredCategory(categoryID)
	local categoryData = AllData[currentCategory]
	local filteredData = {}
	for i=1,#categoryData do
		local visualID = categoryData[i]
		local visualData = Visuals[ visualID ]
		local isFiltered = false
		if visualData then
			for k=1,#visualData do
				local item = visualData[k]				
				local sourceData = ItemSource[item]
				local sourceType = sourceData[1]
				
				if bit_band(SourceTypeBitFilters[sourceType] or SourceTypeBitFilters[0], FilterBits) > 0 and 
				   (not IsFavoritesOnly or VLTW._Fav[visualID]) and 
				   (IsCollectedState == 3 or (IsCollectedState == 1 and (VLTW._Collected[visualID] or VLTW._BOE[visualID])) or (IsCollectedState == 2 and not (VLTW._Collected[visualID] or VLTW._BOE[visualID]))) and
				   (not FilterInstance or (type(FilterInstance)~='table' and sourceType == 1 and CheckFilterInstance(sourceData,1)) or (type(FilterInstance)=='table' and ((sourceType == 2 and CheckFilterInstance(sourceData,2)) or (sourceType == 3 and CheckFilterInstance(sourceData,3)) or (sourceType == 4 and CheckFilterInstance(sourceData,4))))) and
				   (bit_band(FilterDebug,1) == 0 or #sourceData == 1) and
				   (bit_band(FilterDebug,2) == 0 or (sourceType == 1 and CheckFilterDebugNoDiff(sourceData))) then
					isFiltered = true
					break
				end
			end
		end
		if isFiltered then
			filteredData[#filteredData + 1] = visualID
		end
	end
	return filteredData
end

function UpdateCategory(categoryID)
	currentCategory = categoryID
	
	for i=1,#models do
		local model = models[i]
		model.cameraID = CameraData[categoryID]
		model:SetUnit("player")
		model:Undress()
		Model_ApplyUICamera(model, CameraData[categoryID])
	end
	mainFrame.bigModel.cameraID = CameraData[categoryID]
	mainFrame.bigModel.itemID = CategoryBigModelItem[categoryID]
	mainFrame.bigModel:Reload()
	
	IsUsedZeroCam = false
	
	local total = #AllData[categoryID]
	
	if IsFilterEnabled or IsFavoritesOnly or IsCollectedState < 3 or FilterInstance or FilterDebug > 0 then
		AllData[-1] = CreateFilteredCategory(categoryID)
		total = #AllData[-1]
	end
	
	currentPage = 1
	currentMaxPages = ceil(total / #models)
	UpdatePage()
	if isFirstLoad then
		C_Timer.NewTimer(1,function() UpdatePage() end)
	end
	isFirstLoad = nil
	
	local collected = UpdateCollectCategory(categoryID)
	ProgressBar:SetMinMaxValues(0, total)
	ProgressBar:SetValue(collected)
	ProgressBar.text:SetFormattedText(HEIRLOOMS_PROGRESS_FORMAT, collected, total)

end

function FindBonuses(link)
	if link then 
		local bonusesNum,bonusesString = link:match("item:%d+:[0-9%-]*:[0-9%-]*:[0-9%-]*:[0-9%-]*:[0-9%-]*:[0-9%-]*:[0-9%-]*:[0-9%-]*:[0-9%-]*:[0-9%-]*:[0-9%-]*:([0-9%-]*):([0-9:]*)")
		if bonusesNum and bonusesString and tonumber(bonusesNum) then
			local res = {}
			for i=1,tonumber(bonusesNum) do
				local bonus = bonusesString:match("^[0-9%-]+")
				if bonus then
					bonus = tonumber(bonus)
					if majorBonuses[bonus] then
						return bonus
					end
				end
				if bonusesString:find(":") then
					bonusesString = bonusesString:gsub("^[^:]*:","")
				end
			end
		end
	end
	return 0
end
local function table_add(t1,t2)
	for i=1,#t2 do
		t1[#t1+1] = t2[i]
	end
end
local function string_add_or_create(str,toadd)
	if type(str) ~= "string" then
		str = ""
	end
	if not str:find(toadd) then
		str = str .. toadd
	end
	return str
end

local function UpdateBagsData()
	wipe(VLTW[charKey].bags)
	for i=0,4 do
		for j=1,GetContainerNumSlots(i) do
			local link = GetContainerItemLink(i, j)
			if link then
				local itemID = tonumber(link:match("item:(%d+)"))
				if ItemsCheckBonuses[itemID] then
					local bonus = FindBonuses(link)
					
					VLTW[charKey].bags[itemID] = string_add_or_create(VLTW[charKey].bags[itemID],"#"..bonus.."#")
				end
			end
		end
	end
	for _,slot in ipairs {1,2,3,15,5,9,10,6,7,8,11,12,13,14,16,17} do
		local link = GetInventoryItemLink('player',slot)
		if link then
			local itemID = tonumber(link:match("item:(%d+)"))
			if ItemsCheckBonuses[itemID] then
				local bonus = FindBonuses(link)
				
				VLTW[charKey].bags[itemID] = string_add_or_create(VLTW[charKey].bags[itemID],"#"..bonus.."#")
			end
		end
	end
end


local function FindVisual(itemID)
	local visualID = CachedItemToVisual[itemID]
	if visualID then
		return visualID
	end
	for visualID,visualData in pairs(Visuals) do
		for i=1,#visualData do
			if visualData[i] == itemID then
				CachedItemToVisual[itemID] = visualID
				return visualID
			end
		end
	end
	CachedItemToVisual[itemID] = -1
	return -1
end

local function TooltipUpdateByItem(self, link)
	local id = link:match("item:(%d+)")
	if id then
		id = tonumber(id)
		if ItemsCheckBonuses[ id ] then
			local bonus = FindBonuses(link)
			if bonus ~= 0 then
				id = id ..":"..bonus
			end
		end
		local visualID = FindVisual(id)
		if visualID > -1 then
			TooltipLastVisual = visualID
			local collectedStr = VLTW._Collected[visualID] or VLTW._BOE[visualID]
			if collectedStr then
				self:AddLine("Wardrobe: Visual collected by "..collectedStr:gsub(":",", "):gsub("%-"..realmKey,""),1,0.5,1,true)
			else
				self:AddLine("Wardrobe: Visual not collected",1,0.5,1)
			end
			if VLTW._Fav[visualID] then
				self:AddLine("Wardrobe: Favorite appearance",0.5,1,0.5)
			end
			if not IsTooltipTabHelpDisabled and not VLTW._DisableTab then
				self:AddLine("Wardrobe: Press <TAB> button to view all sources",0.533,0.666,1,true)
			end
			return true
		end
	end
end

local function OnTooltipSetItem(self)
	TooltipLastVisual = nil
	if IsTooltipModDisabled then
		return
	end
	local _, link = self:GetItem()
	if link then
		TooltipUpdateByItem(self, link)
	end
end
local function OnTooltipSetTradeSkillItem(self, tradeSkillId, reagentID)
	TooltipLastVisual = nil
	if IsTooltipModDisabled then
		return
	end
	local link
	if not reagentID then
		link = GetTradeSkillItemLink(tradeSkillId)
	else
		link = GetTradeSkillReagentItemLink(tradeSkillId, reagentID)
	end
	if link then
		local isLineAdded = TooltipUpdateByItem(self, link)
		if isLineAdded then
			self:Show()
		end
	end
end

local function OnTooltipHide(self)
	TooltipLastVisual = nil
end

do
	local already = false
	function Mod_Tooltips()
		if already then
			return
		end
		local TipsToModify = {
			"GameTooltip",
			"ItemRefTooltip",
		}
		for _,tipName in pairs(TipsToModify) do
			local tipFrame = _G[tipName]
			tipFrame:HookScript("OnTooltipSetItem",OnTooltipSetItem)
			tipFrame:HookScript("OnHide",OnTooltipHide)
			hooksecurefunc(tipFrame,"SetTradeSkillItem",OnTooltipSetTradeSkillItem)
		end
		already = true
	end
end


do
	local already = false
	local _AuctionFrameBrowse_Update = nil
	function Mod_Auction(disable)
		if not IsAddOnLoaded("Blizzard_AuctionUI") or not IsWindowLoaded then
			return
		end
		if disable then
			AuctionFrameBrowse_Update = _AuctionFrameBrowse_Update
			already = false
			for i=1, NUM_BROWSE_TO_DISPLAY do
				_G["BrowseButton"..i]:SetAlpha(1)
			end
			return
		end
		if already then
			return
		end
		_AuctionFrameBrowse_Update = AuctionFrameBrowse_Update
		AuctionFrameBrowse_Update = function(...)
			local r1,r2,r3 = _AuctionFrameBrowse_Update(...)
			local offset = FauxScrollFrame_GetOffset(BrowseScrollFrame)
			for i=1, NUM_BROWSE_TO_DISPLAY do
				local button = _G["BrowseButton"..i]
				button:SetAlpha(1)
				local link = GetAuctionItemLink("list", offset + i)
				if link then
					local id = link:match("item:(%d+)")
					if id then
						id = tonumber(id)
						if ItemsCheckBonuses[ id ] then
							local bonus = FindBonuses(link)
							if bonus ~= 0 then
								id = id ..":"..bonus
							end
						end
						local visualID = FindVisual(id)
						if visualID > -1 and (VLTW._Collected[visualID] or VLTW._BOE[visualID]) then
							button:SetAlpha(.5)
						end
					end
				end
			end
			return r1,r2,r3
		end
		already = true
	end
end


mainFrame:SetScript("OnShow",function(self)
	self:SetScript("OnShow",nil)
	
	CharQuestsData = GetQuestsCompleted()
	
	LoadAddOn("LegionWardrobeData")
	if _G.LTSData then
		local LTS = _G.LTSData
	
		AllData = LTS.AllData
		CamerasData = LTS.CamerasData
		Visuals = LTS.Visuals
		ItemSource = LTS.ItemSource
		VendorNames = LTS.VendorNames
		InstanceList = LTS.InstanceList
		BossList = LTS.BossList
		DiffList = LTS.DiffList
		ItemCost = LTS.ItemCost
		QuestToZone = LTS.QuestToZone
		ZonesNames = LTS.ZonesNames
		ZoneToDataID = LTS.ZoneToDataID
		DiffToDataID = LTS.DiffToDataID
		QuestZoneToDataID = LTS.QuestZoneToDataID
		NPCToZone = LTS.NPCToZone
		NPCReact = LTS.NPCReact
		
		SetsAllData = LTS.SetsAllData
		Sets = LTS.Sets
		
		AllDataAll = AllData.ALL
		local factionData = AllData[playerFaction] or AllData["Horde"]
		AllData = factionData[playerClass]
		
		local factionData = SetsAllData[playerFaction] or SetsAllData["Horde"]
		SetsAllData = factionData[playerClass]		
		
		LTSData = nil
	else
		self:SetScript("OnShow",function()
			print("<Legion Wardrobe> Error: Legion Wardrobe Data isn't loaded")
			self:Hide()
		end)
		print("<Legion Wardrobe> Error: Legion Wardrobe Data isn't loaded")
		self:Hide()
		return
	end
	
	IsWindowLoaded = true
	eventsFrame:EnableKeyboard(true)
	eventsFrame:SetScript("OnKeyDown",eventsFrame_OnKeyDown)
	if not VLTW._DisableTooltip then
		Mod_Tooltips()
	end
	
	UpdateBagsData()
	
	if not VLTW._Disable then
		for i=1,#CategoriesSlotsByClass do
			UpdateCollectCategory(CategoriesSlotsByClass[i])
		end
	end
	
	UpdateCategory(CategoriesSlotsByClass[1])
	
	if VLTW._EnableAuction then
		Mod_Auction()
	end
end)

function IsItemCollected(itemID)
	local sItemID
	if type(itemID) == 'number' then
		sItemID = itemID
	else
		sItemID = tonumber(itemID:match("^(%d+)"))
	end
	local isCollected = false
	if not ItemsCheckBonuses[sItemID] then
		isCollected = GetItemCount(itemID,true) > 0
		if not isCollected and VLTW[charKey].vs[sItemID] then
			isCollected = true
		end
	else
		local bonusList
		if type(itemID) == 'number' then
			bonusList = {"#0#"}
		else
			bonusList = {strsplit(":", itemID:match("^%d+:(.+)"),nil)}
			for i=1,#bonusList do
				bonusList[i] = "#"..bonusList[i].."#"
			end
		end
		local bags,bank,vs = VLTW[charKey].bags[sItemID], VLTW[charKey].bank[sItemID], VLTW[charKey].vs[sItemID]
		for i=1,#bonusList do
			if bags and bags:find(bonusList[i]) then
				isCollected = true
			end
			if bank and bank:find(bonusList[i]) then
				isCollected = true
			end
			if vs and vs:find(bonusList[i]) then
				isCollected = true
			end
		end
	end
	if not isCollected and VLTW._BOE[itemID] then
		isCollected = true
	end
	
	return isCollected
end

eventsFrame = CreateFrame("Frame")
eventsFrame:RegisterEvent("ADDON_LOADED")
eventsFrame:SetScript("OnEvent",function(self,event,arg1)
	if event == "ADDON_LOADED" then
		if arg1 == GlobalAddonName then
			if type(_G.VLTW) ~= 'table' then
				_G.VLTW = {}
				C_Timer.NewTimer(1,function()
					print("You have installed <Legion Wardrobe> addon. Type \"/wardrobe\" or use button in Collections Journal to open it.")
				end)
			end
			VLTW = _G.VLTW
			if not VLTW[charKey] then
				VLTW[charKey] = {
					bags = {},
					bank = {},
					vs = {},
				}
			end
			
			if not VLTW._BOE then 
				VLTW._BOE = {} 
			end
			
			if not VLTW._Collected then
				VLTW._Collected = {}
			end
			if not VLTW._Fav then
				VLTW._Fav = {}
			end
			
			CameraDataOffset = CamerasRace[playerRace..playerSex] or 0
			
			UpdateAlerts(not VLTW[charKey].visited_VS,not VLTW[charKey].visited_Bank)
			
			if playerClass == "ALL" then
				VLTW._Disable = true
			end
			
			mainFrame.disableOtherChars:SetChecked(VLTW._Disable)
			mainFrame.disableTooltip:SetChecked(VLTW._DisableTooltip)
			mainFrame.enableTabButton:SetChecked(not VLTW._DisableTab)
			mainFrame.enableAuction:SetChecked(VLTW._EnableAuction)
			
			IsTooltipTabHelpDisabled = (VLTW._TABHelpDisable or 0) > 2
			
			self:RegisterEvent("BANKFRAME_OPENED")
			self:RegisterEvent("VOID_STORAGE_OPEN")
			
			if IsAddOnLoaded("Blizzard_Collections") and CollectionsJournal then
				self:GetScript("OnEvent")(self,"ADDON_LOADED","Blizzard_Collections")
			end
		elseif arg1 == "Blizzard_Collections" then
			C_Timer.NewTimer(0.1,function()
				local nextTab = 1
				for i=1,10 do
					if not _G["CollectionsJournalTab"..i] then
						nextTab = i
						break
					end
				end

				local button = CreateFrame("Button","$parentTab"..nextTab,CollectionsJournal,"CollectionsJournalTab")
				button:SetText("Wardrobe")
				button:SetID(nextTab)
				button:SetScript("OnClick",function()
					CollectionsJournal.CloseButton:Click()
					mainFrame:Show()
				end)
				
				local function RePosButton()
					local leftMax,leftTab = 0,1
					for i=1,nextTab-1 do
						local currLeft = _G["CollectionsJournalTab"..i]:GetLeft()
						if currLeft > leftMax then
							leftMax = currLeft
							leftTab = i
						end
					end
					button:SetPoint("LEFT",_G["CollectionsJournalTab"..leftTab],"RIGHT",-16,0)
				end
				RePosButton()
				
				PanelTemplates_DeselectTab(button)
				PanelTemplates_TabResize(button, 0)
				
				C_Timer.NewTimer(1,RePosButton)
			end)
		elseif arg1 == "Blizzard_AuctionUI" then
			if VLTW._EnableAuction then
				Mod_Auction()
			end
		end
	elseif event == "BANKFRAME_OPENED" then
		VLTW[charKey].visited_Bank = true
		mainFrame:Hide()
		wipe(VLTW[charKey].bank)
		for i=5,12 do
			local bug = i==12 and -1 or i
			for j=1,GetContainerNumSlots(bug) do
				local link = GetContainerItemLink(bug, j)
				if link then
					local itemID = tonumber(link:match("item:(%d+)"))
					if ItemsCheckBonuses[itemID] then
						local bonus = FindBonuses(link)
						
						VLTW[charKey].bank[itemID] = string_add_or_create(VLTW[charKey].bank[itemID],"#"..bonus.."#")
					end
				end
			end
		end
		UpdateAlerts(not VLTW[charKey].visited_VS,not VLTW[charKey].visited_Bank)
		if not VLTW._Disable and IsWindowLoaded then
			for i=1,#CategoriesSlotsByClass do
				UpdateCollectCategory(CategoriesSlotsByClass[i])
			end
		end
		self:UnregisterEvent("BANKFRAME_OPENED")
	elseif event == "VOID_STORAGE_OPEN" then
		VLTW[charKey].visited_VS = true
		mainFrame:Hide()
		C_Timer.NewTimer(1,function() 
			wipe(VLTW[charKey].vs)
			for i=1,160 do
				local link = GetVoidItemHyperlinkString(i)
				if link then
					local itemID = tonumber(link:match("item:(%d+)"))
					if ItemsCheckBonuses[itemID] then
						local bonus = FindBonuses(link)
						
						VLTW[charKey].vs[itemID] = string_add_or_create(VLTW[charKey].vs[itemID],"#"..bonus.."#")
					else
						VLTW[charKey].vs[itemID] = true
					end
				end
			end
		end)
		UpdateAlerts(not VLTW[charKey].visited_VS,not VLTW[charKey].visited_Bank)
		if not VLTW._Disable and IsWindowLoaded then
			for i=1,#CategoriesSlotsByClass do
				UpdateCollectCategory(CategoriesSlotsByClass[i])
			end
		end
		self:UnregisterEvent("VOID_STORAGE_OPEN")
	end
end)

function eventsFrame_OnKeyDown(self,button)
	if button == "ESCAPE" then
		if InputFrame and InputFrame:IsShown() then
			return
		end
		if sourcesFrame:IsShown() then
			sourcesFrame:Hide()
			C_Timer.NewTimer(0.01,function() HideUIPanel(GameMenuFrame) end)
			return
		end
		if mainFrame:IsShown() then
			mainFrame:Hide()
			C_Timer.NewTimer(0.01,function() HideUIPanel(GameMenuFrame) end)
			return
		end
	elseif button == "-" or button == "NUMPADMINUS" then
		if mainFrame.bigModel:IsShown() then
			mainFrame.bigModel:GetScript("OnMouseWheel")(mainFrame.bigModel,-1)
		end
	elseif button == "=" or button == "NUMPADPLUS" then
		if mainFrame.bigModel:IsShown() then
			mainFrame.bigModel:GetScript("OnMouseWheel")(mainFrame.bigModel,1)
		end
	elseif button == "TAB" then
		if not GameTooltip:IsShown() or not TooltipLastVisual or VLTW._DisableTab then
			return
		end
		FindVisualAndOpen(TooltipLastVisual)
		
		local tabCount = VLTW._TABHelpDisable or 0
		VLTW._TABHelpDisable = tabCount > 2 and 3 or (tabCount+1)
		IsTooltipTabHelpDisabled = tabCount > 2
	end
end

eventsFrame:SetPropagateKeyboardInput(true)
eventsFrame:SetScript("OnKeyDown",nil)

SlashCmdList["LTWardrobe"] = function(arg)
	if arg and arg:lower() == "boe" then
		if not IsWindowLoaded then
			mainFrame:Show() 
		end
		UpdateCollectStateBOE()
		return
	end
	mainFrame:Show() 
end
SLASH_LTWardrobe1 = "/wardrobe"
SLASH_LTWardrobe2 = "/legionwardrobe"

_G.LWPF = {}

function LWPF:ClearCharData(charName)
	if charName == "" then
		return
	end
	if not charName:find("%-") then
		charName = charName .."-".. realmKey
	end
	local findKey = charName:gsub("%-","%%%-")
	local toRemove = {}
	for visualID,visualData in pairs(VLTW._Collected) do
		if visualData:find(findKey) then
			visualData = strtrim(visualData:gsub(":*"..findKey..":*",":"),":")
			if visualData == "" then
				toRemove[visualID] = true
			else
				VLTW._Collected[visualID] = visualData
			end
		end
	end
	for visualID,_ in pairs(toRemove) do
		VLTW._Collected[visualID] = nil
	end
	print("<Legion Wardrobe>: Char "..charName.." was removed from db")
end




do
	local setsFrameBack = CreateFrame("Button",nil,mainFrame)
	setsFrameBack:Hide()

	setsFrameBack:SetPoint("TOPLEFT")
	setsFrameBack:SetPoint("BOTTOMRIGHT")
	
	setsFrameBack.Texture = setsFrameBack:CreateTexture(nil,"OVERLAY",nil,3)
	setsFrameBack.Texture:SetAllPoints()
	setsFrameBack.Texture:SetTexture(0,0,0,.8)
	
	setsFrameBack:SetScript("OnClick",function() end)
	
	setsFrameBack:SetFrameStrata("DIALOG")

	local setsFrame = CreateFrame("Frame",nil,setsFrameBack)
	setsFrame:SetPoint("CENTER")
	setsFrame:SetSize(585,570)
	
	_CreateShadowBack(setsFrame,true)
	
	setsFrame:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8X8",edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",tile = true,tileSize = 1,edgeSize = 8,insets = {left = 2,right = 2,top = 2,bottom = 2}})
	setsFrame:SetBackdropColor(0.2, 0.2, 0.2, 1) 
	setsFrame:SetBackdropBorderColor(0.25, 0.25, 0.25, 1)
	
	setsFrame.titleText = setsFrame:CreateFontString(nil, "OVERLAY","GameFontWhite", 2)
	setsFrame.titleText:SetFont(setsFrame.titleText:GetFont(),12)
	setsFrame.titleText:SetPoint("TOP",0,-6)
	setsFrame.titleText:SetJustifyH("LEFT")
	setsFrame.titleText:SetJustifyV("TOP")
	setsFrame.titleText:SetText("Sets")
	setsFrame.titleText:SetTextColor(1,.6,0,1)
	
	local closeButton = CreateFrame("Button",nil,setsFrame,"UIPanelCloseButton")
	closeButton:SetPoint("TOPRIGHT",3,2)
	closeButton:SetScript("OnClick",function()
		setsFrameBack:Hide()
	end)
	
	local sourcesFrame = CreateFrame("Frame",GlobalAddonName.."SetsFrameSourcesFrame",UIParent,"UIPanelDialogTemplate")
	sourcesFrame:SetSize(360,300)
	sourcesFrame:SetPoint("CENTER")
	sourcesFrame:SetFrameStrata("DIALOG")
	sourcesFrame:SetClampedToScreen(true)
	sourcesFrame:EnableMouse(true)
	sourcesFrame:SetMovable(true)
	sourcesFrame:RegisterForDrag("LeftButton")
	sourcesFrame:SetScript("OnDragStart", function(self) 
		self:StartMoving() 
	end)
	sourcesFrame:SetScript("OnDragStop", function(self) 
		self:StopMovingOrSizing() 
	end)
	sourcesFrame.title:SetText("Set Visuals")
	sourcesFrame:Hide()
	sourcesFrame:SetToplevel(true)
	sourcesFrame:SetFrameLevel(100)
	
	local function SourcesFrame_MoreInfo_OnClick(self)
		local visualID = self:GetParent().visualID
		FindVisualAndOpen(visualID)
		mainFrame.sourcesFrame:SetFrameLevel(110)
	end

	sourcesFrame.lines = {}
	local function SourceFrame_CreateLine(i)
		if sourcesFrame.lines[i] then
			return sourcesFrame.lines[i]
		end
		local line = CreateFrame("Frame",nil,sourcesFrame)
		sourcesFrame.lines[i] = line
		line:SetPoint("TOPLEFT",10,-30-(i-1)*25)
		line:SetSize(1,20)
		
		line.itemName = line:CreateFontString(nil,"ARTWORK","GameFontWhite")
		line.itemName:SetPoint("LEFT",0,0)
		line.itemName:SetSize(180,20)
		line.itemName:SetJustifyH("LEFT")
		line.itemName:SetJustifyV("MIDDLE")
		
		line.itemNameFrame = CreateFrame("Button",nil,line)
		line.itemNameFrame:SetAllPoints(line.itemName)
		line.itemNameFrame:SetScript("OnEnter",SourcesFrame_ItemName_OnEnter)
		line.itemNameFrame:SetScript("OnLeave",SourcesFrame_HideTooltip)
		
		line.state = line:CreateFontString(nil,"ARTWORK","GameFontWhite")
		line.state:SetPoint("LEFT",line.itemName,"RIGHT",5,0)
		line.state:SetSize(95,20)
		line.state:SetJustifyH("LEFT")
		line.state:SetJustifyV("MIDDLE")
		
		line.moreInfo = CreateFrame("Button",nil,line,"UIPanelButtonTemplate")
		line.moreInfo:SetPoint("LEFT",line.state,"RIGHT",5,0)
		line.moreInfo:SetSize(50,20)
		line.moreInfo:SetText(">>>")
		line.moreInfo:SetScript("OnClick",SourcesFrame_MoreInfo_OnClick)
		
		return line
	end


	local function Model_OnClick(self,button)
		if IsModifiedClick("DRESSUP") then
			for i=1,#self.Equipped do
				DressUpItemLink(self.Equipped[i][1])
			end
			return
		end
		for i=1,#self.Equipped do
			local line = SourceFrame_CreateLine(i)
			
			local data = self.Equipped[i]
			local name,link = GetItemInfo(data[1])
			line.itemName:SetText(link)
			line.itemNameFrame.link = data[1]
			line.visualID = data[2]
			line.state:SetText(data[3] and "|cff00ff00Collected|r" or "|cffff0000Not collected|r") 
			line:Show()
		end
		for i=#self.Equipped+1,#sourcesFrame.lines do
			sourcesFrame.lines[i]:Hide()
		end
		sourcesFrame:SetHeight(40+#self.Equipped*25)
		sourcesFrame:Show()
	end
	
	local function Model_OnShow(self)
		self:SetUnit("player")
		self:Undress()
		if self.Equipped then
			for i=1,#self.Equipped do
				self:TryOn(self.Equipped[i][1])
			end
		end
	end
	
	local function Model_OnEnter(self)
		self.border:SetBackdropBorderColor(1, 1, 1, 1)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:AddLine(self.setData[1]..(IsDebugVersion and " SetID:"..self.setID or ""))
		if self.setData[2] then
			local titles = strjoin(", ", unpack(self.setData[2]))
			GameTooltip:AddLine(titles,1,1,1,true)
		end
		for i=1,#self.Equipped do
			local data = self.Equipped[i]
			local name,link = GetItemInfo(data[1])
			GameTooltip:AddDoubleLine(link or " ",data[3] and "|cff00ff00Collected|r" or "|cffff0000Not collected|r")
		end
		GameTooltip:Show()
	end
	
	local function Model_OnLeave(self)
		if self.IsCollected then
			self.border:SetBackdropBorderColor(0, 1, 0, 1)
		else
			self.border:SetBackdropBorderColor(0.6, 0.6, 0.6, .8)
		end
		GameTooltip:Hide()
	end
	
	local models = {}
	for i=0,1 do
		for j=0,2 do
			local borderFrame = CreateFrame("Frame",nil,setsFrame)
		
			local model = CreateFrame("DressUpModel",nil,setsFrame)
			models[i*3+j+1] = model
			model:SetPoint("TOPLEFT",40+175*j,-30-250*i)
			model:SetSize(150,200)
			
			model.Equipped = {}
			
			model:SetScript("OnShow",Model_OnShow)
			model:SetScript("OnEnter",Model_OnEnter)
			model:SetScript("OnLeave",Model_OnLeave)
			model:SetScript("OnMouseUp",Model_OnClick)
			
			borderFrame:SetPoint("TOPLEFT",model,-4,4)
			borderFrame:SetPoint("BOTTOMRIGHT",model,4,-4)
			
			borderFrame:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8X8",edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",tile = true,tileSize = 1,edgeSize = 8,insets = {left = 2,right = 2,top = 2,bottom = 2}})
			borderFrame:SetBackdropColor(0.08, 0.08, 0.08, 1) 
			borderFrame:SetBackdropBorderColor(0.6, 0.6, 0.6, 1)
			
			model.border = borderFrame
			
			local ProgressBar = CreateFrame("StatusBar",nil,model)
			ProgressBar:SetSize(130,13)
			ProgressBar:SetPoint("TOP",model,"BOTTOM",0,-10)
			ProgressBar:SetStatusBarTexture([[Interface\PaperDollInfoFrame\UI-Character-Skills-Bar]]) 
			ProgressBar:SetStatusBarColor(0.03125,0.85,0.0)
			do
				local borderLeft = ProgressBar:CreateTexture(nil,"OVERLAY")
				borderLeft:SetPoint("LEFT",-3,0)
				borderLeft:SetSize(8,17)
				borderLeft:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-Skills-BarBorder]])
				borderLeft:SetTexCoord(0.007843,0.043137,0.193548,0.774193)
			
				local borderRight = ProgressBar:CreateTexture(nil,"OVERLAY")
				borderRight:SetPoint("RIGHT",4,0)
				borderRight:SetSize(8,17)
				borderRight:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-Skills-BarBorder]])
				borderRight:SetTexCoord(0.043137,0.007843,0.193548,0.774193)
			
				local borderMid = ProgressBar:CreateTexture(nil,"OVERLAY")
				borderMid:SetPoint("TOPLEFT",borderLeft,"TOPRIGHT")
				borderMid:SetPoint("BOTTOMRIGHT",borderRight,"BOTTOMLEFT")
				borderMid:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-Skills-BarBorder]])
				borderMid:SetTexCoord(0.113726,0.1490196,0.193548,0.774193)
				
				local Background = ProgressBar:CreateTexture(nil,"BACKGROUND",nil,-1)
				Background:SetAllPoints()
				Background:SetTexture(0,0,0,1)
				
				local text = ProgressBar:CreateFontString(nil,"OVERLAY","TextStatusBarText",1)
				text:SetPoint("CENTER",0,0)
				text:SetSize(0,9)
				text:SetJustifyH("CENTER")
				ProgressBar.text = text
			end
			
			model.ProgressBar = ProgressBar
		end
	end
	
	local currentPage, currentMaxPages = 1, 1
	
	local function UpdateSetModel(model,setID)
		model:Show()
		model.border:Show()
		model:Undress()
		
		wipe(model.Equipped)
		
		local collected = 0
		
		local setData = Sets[setID]
		for i=3,#setData do
			local visualID = setData[i]
			local visualData = Visuals[ visualID ]
			local item = visualData[1]
			local itemID = type(item) == 'number' and item or tonumber(item:match("^(%d+)"))
			local itemLink = "item:"..itemID
			
			if type(item) == 'string' then
				local count = 0
				for j=1,#item do
					if item:sub(j,j) == ":" then
						count = count + 1
					end
				end
				itemLink = itemLink .. ":0:0:0:0:0:0:0:"..UnitLevel'player'..":0:0:0:"..count..":"..item:gsub("^%d+:","")
			end
			
			local isCollected = false
			if VLTW._Collected[visualID] or VLTW._BOE[visualID] then
				collected = collected + 1
				isCollected = true
			end
			
			GetItemInfo(itemLink)	--Cache it
			
			model:TryOn(itemLink)
			model.Equipped[#model.Equipped + 1] = {itemLink,visualID,isCollected}
		end
		model.setData = setData
		model.setID = setID
		
		local total = #setData - 2
		
		model.ProgressBar:SetMinMaxValues(0, total)
		model.ProgressBar:SetValue(collected)
		model.ProgressBar.text:SetFormattedText(HEIRLOOMS_PROGRESS_FORMAT, collected, total)
		
		model.IsCollected = collected == total
		Model_OnLeave(model)
	end
	
	local UpdateSetPage
	
	local PageText = setsFrame:CreateFontString(nil,"ARTWORK","GameFontWhite")
	PageText:SetPoint("BOTTOM",-57,33)
	PageText:SetJustifyH("RIGHT")
	
	local PrevPageButton = CreateFrame("Button",nil,setsFrame)
	PrevPageButton:SetPoint("BOTTOM",20,23)
	PrevPageButton:SetSize(32,32)
	PrevPageButton:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up") 
	PrevPageButton:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Down") 
	PrevPageButton:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Disabled") 
	PrevPageButton:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight","ADD") 
	PrevPageButton:SetScript("OnClick",function ()
		currentPage = max(1,currentPage - 1)
		UpdateSetPage()
	end)
	
	local NextPageButton = CreateFrame("Button",nil,setsFrame)
	NextPageButton:SetPoint("BOTTOM",55,23)
	NextPageButton:SetSize(32,32)
	NextPageButton:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up") 
	NextPageButton:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down") 
	NextPageButton:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Disabled") 
	NextPageButton:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight","ADD")
	NextPageButton:SetScript("OnClick",function ()
		currentPage = min(currentMaxPages,currentPage + 1)
		UpdateSetPage()
	end)
	
	setsFrame:SetScript("OnMouseWheel",function (self,delta)
		if delta < 0 and NextPageButton:IsEnabled() then
			NextPageButton:Click()
		elseif delta > 0 and PrevPageButton:IsEnabled() then
			PrevPageButton:Click()
		end
	end)
	
	function UpdateSetPage()
		GameTooltip_Hide()
	
		PageText:SetFormattedText(COLLECTION_PAGE_NUMBER, currentPage, currentMaxPages)
		if currentPage == 1 then PrevPageButton:Disable() else PrevPageButton:Enable() end
		if currentPage >= currentMaxPages then NextPageButton:Disable() else NextPageButton:Enable() end
	
		local pos = (currentPage - 1) * #models + 1
		local m_c = 0
		
		for i=pos,#SetsAllData do
			m_c = m_c + 1
			local model = models[m_c]
			
			UpdateSetModel(model, SetsAllData[i])
			
			if m_c >= #models then
				break
			end
		end
		for i=m_c+1,#models do
			models[i]:Hide()
			models[i].border:Hide()
		end
	end
	
	setsFrame:SetScript("OnShow",function(self)
		self:SetScript("OnShow",nil)
		
		for i=1,6 do
			local model = models[i]
			model:SetUnit("player")
			model:Undress()
		end
		
		local total = #SetsAllData
		
		currentPage = 1
		currentMaxPages = ceil(total / #models)
		UpdateSetPage()
	end)

	local function ButtonOnEnter(self)
		if not self.tooltip then return end
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:AddLine(self.tooltip)
		GameTooltip:Show()
	end
	local function ButtonOnLeave(self)
		GameTooltip_Hide()
	end
	local function ButtonOnClick(self)
		setsFrameBack:Show()
	end

	local button = CreateFrame("Button",nil,mainFrame)
	button:SetPoint("BOTTOMRIGHT",-50,13)
	button:SetSize(31,31)
	button:SetScript("OnClick",ButtonOnClick)
	button:SetScript("OnEnter",ButtonOnEnter)
	button:SetScript("OnLeave",ButtonOnLeave)
	button.tooltip = "Sets"
	
	local normalTexture = button:CreateTexture(nil,"BACKGROUND")
	normalTexture:SetAllPoints()
	normalTexture:SetTexture([[Interface\AddOns\LegionWardrobe\Transmogrify]])
	normalTexture:SetTexCoord(231/512,266/512,48/512,85/512)
	
	local highlightTexture = button:CreateTexture(nil,"BACKGROUND")
	highlightTexture:SetSize(31,31)
	highlightTexture:SetAtlas("bags-roundhighlight")
	highlightTexture:SetPoint("CENTER",0,1)
	
	button:SetNormalTexture(normalTexture) 
	button:SetHighlightTexture(highlightTexture,"ADD") 
end